@@ -54,33 +54,3 @@ void Smb4KAuthInfo::setShare( const QString &share )
 }
 
 
-QString Smb4KAuthInfo::maskedPassword() const
-{
-  QString p = m_password.stripWhiteSpace();
-  
-  // Replace special characters.
-  p.replace( "\\", "\\\\" );
-  p.replace( "'", "\\'" );
-  p.replace( "$", "\\$" );
-  p.replace( "#", "\\#" );
-  p.replace( "\"", "\\\"" );
-  p.replace( "&", "\\&" );
-  p.replace( ";", "\\;" );
-  p.replace( "<", "\\<" );
-  p.replace( ">", "\\>" );
-  p.replace( "|", "\\|" );
-  p.replace( "?", "\\?" );
-  p.replace( "(", "\\(" );
-  p.replace( ")", "\\)" );
-  p.replace( "[", "\\[" );
-  p.replace( "]", "\\]" );
-  p.replace( "{", "\\{" );
-  p.replace( "}", "\\}" );
-  p.replace( "'", "\\'" );
-  p.replace( "`", "\\`" );
-  p.replace( "*", "\\*" );
-  p.replace( "", "\\" );
-  p.replace( "%", "\\%" );
-  
-  return p;
-}
