@@ -18,7 +18,8 @@
  //Application includes
 #include "smb4kbookmark.h"
 
-Smb4KBookmark::Smb4KBookmark(const QString &share) : m_share(share)
+Smb4KBookmark::Smb4KBookmark( const QString &share, const QString &type ) 
+: m_share(share), m_type( type )
 {
     //FIXME should throw an exception if one of the param is empty
 }
@@ -29,6 +30,12 @@ Smb4KBookmark::~Smb4KBookmark(){}
 
 QString Smb4KBookmark::getShare() const
 {
-    return m_share;
+  return m_share;
+}
+
+
+QString Smb4KBookmark::getType() const
+{
+  return m_type;
 }
 
