@@ -24,63 +24,80 @@
 
 // Qt includes
 #include <qobject.h>
-#include <qpopupmenu.h>
-#include <qptrlist.h>
+#include <qvaluelist.h>
 
 // KDE includes
 #include <kconfig.h>
-#include <klistview.h>
 
 // Application includes
 #include "smb4kbookmark.h"
 
-/** This class belongs the to core classes of Smb4K and manages the
-    bookmarks. */
+/** 
+ * This class belongs the to core classes of Smb4K and manages the
+ * bookmarks. 
+ */
 
 class Smb4KBookmarkHandler : public QObject
 {
   Q_OBJECT
+  
+  friend class Smb4KCore;
 
   public:
-    /** The constructor. */
+    /** 
+     * The constructor. 
+     */
     Smb4KBookmarkHandler( QObject *parent = 0, const char *name = 0 );
-    /** The destructor. */
+    /** 
+     * The destructor. 
+     */
     ~Smb4KBookmarkHandler();
-
-    /** Writes a bookmark to the bookmark file. */
-    void writeBookmark( const QString &host, const QString &share );
-    /** Load the bookmarks. */
+    /**
+     * Writes a bookmark to the bookmark file. This works essentially
+     * like the function above, but it takes a Smb4KBookmark object.
+     */
+    void addBookmark( Smb4KBookmark *bookmark );
+    /**
+     * This function writes a new list of bookmarks. The old list will be
+     * deleted.
+     */
+    void writeBookmarkList( const QValueList<Smb4KBookmark *> &list );
+    /** 
+     * Load the bookmarks. 
+     */
     void loadBookmarks();
-    /** Get the list of bookmarks, */
+    /** 
+     * Get the list of bookmarks. 
+     */
     const QValueList<Smb4KBookmark *> &getBookmarks() { return m_bookmarks; }
-    /** Invokes the bookmark editor. */
-    void editBookmarks( QWidget *parent = 0, const char *name = 0 );
-
+    
   signals:
-    /** Signal emitted when the list of bookmarks has been updated. */
+    /** 
+     * Signal emitted when the list of bookmarks has been updated. 
+     */
     void bookmarksUpdated();
-
-  protected slots:
-    /** Is invoked, when the OK button on the editor is clicked. */
-    void slotOKClicked();
-    /** Is invoked if the right mouse button has been pressed in the editor. */
-    void slotRightButtonPressed( QListViewItem *item, const QPoint &pos, int col );
-    /** Removes an item from the bookmark editor. */
-    void slotRemoveClicked();
-    /** Removes all items from the bookmark editor. */
-    void slotRemoveAllClicked();
-
+    /**
+     * This signal is emitted, when an error occurred.
+     *
+     * @param error_code  The error code as defined in smb4kdefs.h.
+     *
+     * @param error_message The error message. Default is <code>QString::null</code>.
+     */
+    void error( int error_code, const QString &error_message = QString::null );
+    
   private:
-    /** The KConfig object, that handles the bookmark file. */
+    /** 
+     * The KConfig object, that handles the bookmark file. 
+     */
     KConfig *m_config;
-    /** Holds the number of bookmarks in the file. */
-    int m_num;
-    /** The edit dialog. */
-    KDialogBase *m_editDlg;
-    /** The list view object. */
-    KListView *m_bmView;
-    /** The list of bookmarks. */
+    /** 
+     * The list of bookmarks. 
+     */
     QValueList<Smb4KBookmark *> m_bookmarks;
+    /**
+     * Removes all bookmarks.
+     */
+    void clear();
 };
 
 #endif
