@@ -19,12 +19,14 @@
 #include <qlayout.h>
 #include <qlabel.h>
 #include <qcheckbox.h>
+#include <qgroupbox.h>
 
 #ifdef __FreeBSD__
 #include <qdir.h>
 #include <qfile.h>
 #include <qstringlist.h>
 #include <qurlinfo.h>
+#include <qmap.h>
 #endif
 
 // KDE includes
@@ -45,14 +47,17 @@
 #include "smb4kcore.h"
 
 
-Smb4KPasswdReader::Smb4KPasswdReader( QWidget *parent, const char *name ) : QObject( parent, name )
+Smb4KPasswdReader::Smb4KPasswdReader( QObject *parent, const char *name ) : QObject( parent, name )
 {
   // First we need the directory.
   KStandardDirs *stddir = new KStandardDirs();
   QString dir = locateLocal( "appdata", QString::null, KGlobal::instance() );
+  
   if ( !stddir->exists( dir ) )
     stddir->makeDir( dir );
-
+  
+  delete stddir;
+    
   // Do not use the KConfig object, that is provided by kapp->config().
   // We want a separate one for the passwords.
   m_config = new KConfig( "passwords", false, true, "appdata" );
@@ -77,9 +82,7 @@ Smb4KPasswdReader::Smb4KPasswdReader( QWidget *parent, const char *name ) : QObj
 
 Smb4KPasswdReader::~Smb4KPasswdReader()
 {
-  // Delete all pointers in the auth data list.
-  for ( QValueList<Smb4KAuthInfo *>::Iterator it = m_list.begin(); it != m_list.end(); ++it )
-    delete *it;
+  m_list.clear();
 }
 
 
@@ -97,7 +100,7 @@ Smb4KAuthInfo *Smb4KPasswdReader::getAuth( const QString &workgroup, const QStri
     {
       if ( !workgroup.isEmpty() )
       {
-        if ( (*it)->workgroup() == workgroup && (*it)->host() == host )
+        if ( (*it)->workgroup().upper() == workgroup.upper() && (*it)->host().upper() == host.upper() )
         {
           if ( !share.isEmpty() )
           {
@@ -116,7 +119,9 @@ Smb4KAuthInfo *Smb4KPasswdReader::getAuth( const QString &workgroup, const QStri
               continue;            
             }
             else
+            {
               continue;
+            }
           }
           else
           {
@@ -128,26 +133,36 @@ Smb4KAuthInfo *Smb4KPasswdReader::getAuth( const QString &workgroup, const QStri
               break;
             }
             else
+            {
               continue;
+            }
           }
         }
         else
+        {
           continue;
+        }
       }
       else
       {
-        if ( (*it)->host() == host )
+        if ( (*it)->host().upper() == host.upper() )
         {
           auth->setUser( (*it)->user() );
           auth->setPassword( (*it)->password() );
           
           if ( (*it)->share() == "*" )
+          {
             break;
+          }
           else
+          {
             continue;
+          }
         }
         else
+        {
           continue;
+        }
       }
     }
   }
@@ -183,8 +198,14 @@ void Smb4KPasswdReader::read()
 {
   if ( m_config->hasGroup( "Logins" ) )
   {
+    if ( !m_list.isEmpty() )
+    {
+      m_list.clear();
+    }
+  
     QStringList loginData;
     m_config->setGroup( "Logins" );
+    
     for ( int pos = 0; ; pos++ )
     {
       loginData = m_config->readListEntry( QString( "%1" ).arg( pos, 0, 10 ).stripWhiteSpace(), ',' );
@@ -248,8 +269,7 @@ void Smb4KPasswdReader::commit( Smb4KAuthInfo *auth )
 
 
 /****************************************************************************
-   Writes the auth data that is passed by the configuration dialog on
-   exit.
+   Writes the list of authentication data to the password file.
 ****************************************************************************/
 
 void Smb4KPasswdReader::write()
@@ -264,12 +284,20 @@ void Smb4KPasswdReader::write()
   m_config->setGroup( "Logins" );
   
   int index = 0;
-  
+
+  QStringList item;
+    
   for ( QValueList<Smb4KAuthInfo *>::ConstIterator it = m_list.begin(); it != m_list.end(); ++it )
   {
-    QStringList item;
-    item << (*it)->workgroup() << (*it)->host() << (*it)->share() << (*it)->user() << obfuscate( (*it)->password(), true );
+    item.append( (*it)->workgroup() );
+    item.append( (*it)->host() );
+    item.append( (*it)->share() );
+    item.append( (*it)->user() );
+    item.append(  obfuscate( (*it)->password(), true ) );
+    
     m_config->writeEntry( QString( "%1" ).arg( index++ ), item, ',' );
+    
+    item.clear();
   }
   
   m_config->sync();
@@ -303,179 +331,180 @@ void Smb4KPasswdReader::writeDefaultAuthData( Smb4KAuthInfo *auth )
    This function shows the askpass dialog.
 ****************************************************************************/
 
-bool Smb4KPasswdReader::askpass( const QString &workgroup, const QString &host, const QString &share )
+bool Smb4KPasswdReader::askpass( const QString &workgroup, const QString &host, const QString &share, int state )
 {
-  passwdDlg = new KDialogBase( KDialogBase::Plain, i18n( "Authentication" ), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, (QWidget *)parent(), "PasswordDlg", false, true );
+  //
+  // Build the widget.
+  //
+  passwdDlg = new KDialogBase( KDialogBase::Plain, i18n( "Authentication" ), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, new QWidget(), "PasswordDlg", false, true );
 
-  //
-  // Set up the ask pass dialog.
-  //
   QFrame *frame = passwdDlg->plainPage();
   QGridLayout *layout = new QGridLayout( frame );
-  layout->setSpacing( 5 );
-
-  QLabel *pic = new QLabel( frame );
-  pic->setPixmap( DesktopIcon( "personal" ) );
-  pic->setMargin( 10 );
+  layout->setSpacing( 10 );
+  layout->setMargin( 0 );
   
-  layout->addWidget( pic, 0, 0, 0 );
-
-  QWidget *shareData = new QWidget( frame );
-  QGridLayout *grid = new QGridLayout( shareData );
-  grid->setSpacing( 5 );
-  grid->setMargin( 0 );
-
-  QLabel *label1 = new QLabel( i18n( "Workgroup:" ), shareData );
-  QLabel *workgroupLabel = new QLabel( shareData, "Workgroup" );
-  QLabel *label2 = new QLabel( i18n( "Host:" ), shareData );
-  QLabel *hostLabel = new QLabel( shareData, "Host" );
-  QLabel *label3 = new QLabel( i18n( "Share:" ), shareData );
-  QLabel *shareLabel = new QLabel( shareData );
-
-  grid->addWidget( label1, 1, 0, 0 );
-  grid->addWidget( workgroupLabel, 1, 1, 0 );
-  grid->addWidget( label2, 2, 0, 0 );
-  grid->addWidget( hostLabel, 2, 1, 0 );
-  grid->addWidget( label3, 3, 0, 0 );
-  grid->addWidget( shareLabel, 3, 1, 0 );
+  QString message;
   
-  layout->addMultiCellWidget( shareData, 0, 0, 1, 3, 0 );
+  switch ( state )
+  {
+    case NewData:
+      break;
+    case AccessDenied:
+      message = i18n( "The access was denied. " );
+      break;
+    case BadPasswd:
+      message = i18n( "The password is not correct. " );
+      break;
+    case PermDenied:
+      message = i18n( "The permission was denied. " );
+      break;
+    case AuthError:
+      message = i18n( "An authentication error occurred. " );
+      break;
+    case LogonFailure:
+      message = i18n( "The logon failed. " );
+      break;
+    default:
+      break;
+  }    
 
-  QLabel *user = new QLabel( i18n( "User:" ), frame );
+  message.append( i18n( "Please enter authentication data for %1." ).arg( share.stripWhiteSpace().isEmpty() ? host : "//"+host+"/"+share ) );
   
-  layout->addWidget( user, 1, 0, 0 );
- 
-  // This is perhaps a workaround, but it works...
-  KLineEdit *userEdit = new KLineEdit( frame );
-  KComboBox *userCombo = new KComboBox( frame, "Combo" );
- 
-    
+  QGroupBox *message_box = new QGroupBox( 1, Qt::Horizontal, i18n( "Information" ), frame, "AskPassInfoBox" );
+  QLabel *message_label = new QLabel( message_box );
+  message_label->setText( message.stripWhiteSpace() );
+  message_label->setTextFormat( Qt::RichText );
+  
+  layout->addWidget( message_box, 0, 0, 0 ); 
+  
+  QGroupBox *input_box = new QGroupBox( 1, Qt::Horizontal, i18n( "Input" ), frame, "AskPassInputBox" );
+  QWidget *input_widget = new QWidget( input_box );
+  QGridLayout *iw_layout = new QGridLayout( input_widget );
+  iw_layout->setSpacing( 10 );
+  iw_layout->setMargin( 0 );
+  
+  QLabel *userLabel = new QLabel( i18n( "User:" ), input_widget );
+  iw_layout->addWidget( userLabel, 0, 0, 0 );
+  
+  KLineEdit *userEdit = NULL;
+  KComboBox *userCombo = NULL;
+
   if ( share != "homes" )
   {
-    // We don't need this anymore.
-    delete userCombo;    
-    
-    layout->addMultiCellWidget( userEdit, 1, 1, 1, 4, 0 );
+    delete userCombo;
+    userEdit = new KLineEdit( input_widget );
+    iw_layout->addWidget( userEdit, 0, 1, 0 );
   }
   else
   {
-    // We don't need this anymore.
     delete userEdit;
-    
+    userCombo = new KComboBox( input_widget );
     userCombo->setEditable( true );
-    layout->addMultiCellWidget( userCombo, 1, 1, 1, 4, 0 );
+    iw_layout->addWidget( userCombo, 0, 1, 0 );
   }
   
-  QLabel *pass = new QLabel( i18n( "Password:" ), frame );
-  layout->addWidget( pass, 2, 0, 0 );
+  QLabel *passwdLabel = new QLabel( i18n( "Password:" ), input_widget );
+  iw_layout->addWidget( passwdLabel, 1, 0, 0 );
   
-  m_passwdEdit = new KLineEdit( frame );
+  m_passwdEdit = new KLineEdit( input_widget );
   m_passwdEdit->setEchoMode( KLineEdit::Password );
-  layout->addMultiCellWidget( m_passwdEdit, 2, 2, 1, 4, 0 );
-
-  QCheckBox *use = new QCheckBox( i18n( "Use authentication for all shares." ), frame );
-  layout->addMultiCellWidget( use, 3, 3, 0, 3, 0 );
-
-  QSpacerItem *spacer1 = new QSpacerItem( 10, 10, QSizePolicy::Expanding, QSizePolicy::Preferred );
-  layout->addItem( spacer1, 0, 4 );
-
-  //
-  // Data processing and completion of the dialog.
-  //
- 
-  if ( !workgroup.isEmpty() )
-    workgroupLabel->setText( workgroup );
-  else
-    workgroupLabel->setText( "---" );
-    
-  hostLabel->setText( host );
+  iw_layout->addWidget( m_passwdEdit, 1, 1, 0 );
   
-  if ( share.isEmpty() )
-  {
-    use->setEnabled( false );
-    use->setChecked( true );
-    shareLabel->setText( "---" );
-  }
-  else
-  {
-    use->setEnabled( true );
-    shareLabel->setText( share );
-  }
+  QCheckBox *use = new QCheckBox( i18n( "Use authentication for all shares." ), input_box );
   
-  // The new authentication information:
+  layout->addWidget( input_box, 1, 0, 0 );
+  
+  message_box->setFixedWidth( input_box->sizeHint().width() );  // Do not change this! You'll get layout probems otherwise.
+  
+  passwdDlg->setMainWidget( frame );
+  
+  //
+  // Get the authentication info needed.
+  //
   Smb4KAuthInfo *newAuth = new Smb4KAuthInfo( workgroup, host, share );
-    
-  // List for the logins, if the specified share was a 'homes'
-  // share.
-  QStringList logins;
-    
+  
+  QStringList user_names;
+  
   if ( share != "homes" )
   {
-    // Get the auth data.
     Smb4KAuthInfo *auth = getAuth( workgroup, host, share );
- 
-    // If this share is one derived from a homes share we
-    // have to get the user name from the config file.     
+
     if ( auth->user().isEmpty() && auth->password().isEmpty() )
     {
       if ( kapp->config()->hasGroup( "Homes Shares" ) )
       {
         kapp->config()->setGroup( "Homes Shares" );
         QString name = *( kapp->config()->readListEntry( host, ',' ).find( share ) );
+        
         if ( !name.isEmpty() )
+        {
           auth->setUser( name );
+        }
       }
     }
-       
-    // Fill the line edits:
+
     if ( !auth->user().isEmpty() )
+    {
       userEdit->setText( auth->user() );
-       
+    }
+     
     if ( !auth->password().isEmpty() )
+    {
       m_passwdEdit->setText( auth->password() );
-        
-    // Set the cursor to the right line edit:
+    }
+
     if ( userEdit->text().stripWhiteSpace().isEmpty() || ( !userEdit->text().stripWhiteSpace().isEmpty() && !m_passwdEdit->text().stripWhiteSpace().isEmpty() ) )
+    {
       userEdit->setFocus();
+    }
     else
+    {
       m_passwdEdit->setFocus();
+    }
   }
   else
   {
-    // Read the list of logins, that were already defined
-    // for this 'homes' share.
     if ( kapp->config()->hasGroup( "Homes Shares" ) )
     {
       kapp->config()->setGroup( "Homes Shares" );
+
       if ( kapp->config()->hasKey( host ) )
-        logins = kapp->config()->readListEntry( host, ',' );
+      {
+        user_names = kapp->config()->readListEntry( host, ',' );
+      }
     }
 
-    if ( !logins.isEmpty() )
-      userCombo->insertStringList( logins, 0 );
+    if ( !user_names.isEmpty() )
+    {
+      userCombo->insertStringList( user_names, 0 );
+    }
 
     userCombo->setCurrentText( QString::null );
-
-    // Do the last things before showing.
     userCombo->setFocus();
-    
+   
     connect( userCombo, SIGNAL( activated( const QString & ) ), SLOT( slotGetPassword( const QString & ) ) );
   }
   
-  passwdDlg->setFixedSize( passwdDlg->sizeHint() );
+  //
+  // Show the dialog and process the user supplied authentication
+  // data.
+  //
+  bool new_password = false;
+  passwdDlg->setInitialSize( passwdDlg->sizeHint(), true );
   
-  bool new_passwd = false;
-    
-  // Show it.
   if ( passwdDlg->exec() == KDialogBase::Accepted )
   {
     if ( share != "homes" )
     {
       if ( share.isEmpty() || use->isChecked() )
+      {
         newAuth->setShare( "*" );
+      }
       else
+      {
         newAuth->setShare( share );
-        
+      }
+
       // Since we want to allow the user to provide an empty password,
       // only the user name is checked to be not empty.
       if ( !userEdit->text().stripWhiteSpace().isEmpty() )
@@ -483,36 +512,35 @@ bool Smb4KPasswdReader::askpass( const QString &workgroup, const QString &host,
         newAuth->setUser( userEdit->text().stripWhiteSpace() );
         newAuth->setPassword( m_passwdEdit->text().stripWhiteSpace() );
         commit( newAuth );     
-        new_passwd = true;
       }
     }
     else
     {
-      // Same as above:
       if ( !userCombo->currentText().stripWhiteSpace().isEmpty() )
       {
         // Save the list of users to the config file.
         kapp->config()->setGroup( "Homes Shares" );
-        
-        if ( logins.contains( userCombo->currentText().stripWhiteSpace() ) == 0 )
-          logins.append( userCombo->currentText().stripWhiteSpace() );
-          
-        kapp->config()->writeEntry( host, logins, ',' );
-          
+   
+        if ( user_names.contains( userCombo->currentText().stripWhiteSpace() ) == 0 )
+        {
+          user_names.append( userCombo->currentText().stripWhiteSpace() );
+        }   
+
+        kapp->config()->writeEntry( host, user_names, ',' );
+     
         // Put the share in the authentication data list:
         newAuth->setShare( userCombo->currentText().stripWhiteSpace() );
-        
+         
         // Save the authentication data:
         newAuth->setUser( userCombo->currentText().stripWhiteSpace() );
         newAuth->setPassword( m_passwdEdit->text().stripWhiteSpace() );
         commit( newAuth );
-        
-        new_passwd = true;
       }
     }
-  }  
+    new_password = true;
+  }
   
-  return new_passwd;
+  return new_password;
 }
 
 
@@ -522,6 +550,7 @@ bool Smb4KPasswdReader::askpass( const QString &workgroup, const QString &host,
 
 void Smb4KPasswdReader::updateAuthList( const QValueList<Smb4KAuthInfo *> &list )
 {
+  m_list.clear();
   m_list = list;
   
   write();
@@ -569,6 +598,9 @@ void Smb4KPasswdReader::writeToSMBConfFile()
 {
   QDir::setCurrent( QDir::homeDirPath() );
   
+  QString wins = ((Smb4KCore *)parent())->getWINSServer();
+  QMap<QString, QString> options = ((Smb4KCore *)parent())->getGlobalSambaOptions();
+  
   QFile file( ".nsmbrc" );
   
   QStringList contents;
@@ -589,15 +621,15 @@ void Smb4KPasswdReader::writeToSMBConfFile()
     {
       contents.append( "[default]" );
       
-      kapp->config()->setGroup( "Browse Options" );
-      
-      if ( !kapp->config()->readEntry( "WINS Server" ).isEmpty() )
-        contents.append( "nbns="+kapp->config()->readEntry( "WINS Server" ) );
-        
-      QMap<QString, QString> options = ((Smb4KCore *)parent())->fileIO()->getSMBOptions();
+      if ( !wins.isEmpty() )
+      {
+        contents.append( "nbns="+wins );
+      }
       
       if ( !options["workgroup"].isEmpty() )
+      {
         contents.append( "workgroup="+options["workgroup"] );      
+      }
         
       contents.append( QString::null );
     }    
@@ -606,15 +638,15 @@ void Smb4KPasswdReader::writeToSMBConfFile()
   {
     contents.append( "[default]" );
     
-    kapp->config()->setGroup( "Browse Options" );
-    
-    if ( !kapp->config()->readEntry( "WINS Server" ).isEmpty() )
-      contents.append( "nbns="+kapp->config()->readEntry( "WINS Server" ) );
+    if ( !wins.isEmpty() )
+    {
+      contents.append( "nbns="+wins );
+    }
       
-    QMap<QString, QString> options = ((Smb4KCore *)parent())->fileIO()->getSMBOptions();
-    
     if ( !options["workgroup"].isEmpty() )
+    {
       contents.append( "workgroup="+options["workgroup"] );
+    }
       
     contents.append( QString::null );
   }
@@ -634,7 +666,12 @@ void Smb4KPasswdReader::writeToSMBConfFile()
       if ( contents.grep( "["+(*it)->host()+":"+(*it)->user()+":"+(*it)->share()+"]", false ).count() == 0 )
       {
         contents.append( "["+(*it)->host().upper()+":"+(*it)->user().upper()+":"+(*it)->share().upper()+"]" );
-        contents.append( "workgroup="+(*it)->workgroup().upper() );
+        
+        if ( !(*it)->workgroup().isEmpty() )
+        {
+          contents.append( "workgroup="+(*it)->workgroup().upper() );
+        }
+        
         contents.append( "password="+(*it)->password() );
         contents.append( QString::null );      
       }
@@ -650,7 +687,9 @@ void Smb4KPasswdReader::writeToSMBConfFile()
             QString pwd = (*i).section( "=", 1, 1 ).stripWhiteSpace();
             
             if ( pwd != (*it)->password() )
+	    {
               (*i).replace( pwd, (*it)->password() );
+	    }
           }
           
           ++i;
