@@ -17,6 +17,7 @@
 
 // Qt includes
 #include <qapplication.h>
+#include <qmap.h>
 
 // KDE includes
 #include <klocale.h>
@@ -76,54 +77,72 @@ Smb4KScanner::~Smb4KScanner()
 
 void Smb4KScanner::m_init()
 {
+  m_config->setGroup( "Browse Options" );
+  bool auth_for_browse_list = m_config->readBoolEntry( "Auth For Browse List", false );
+  QString method = m_config->readEntry( "Browse List", "nmblookup" );
+  QString host_to_query = m_config->readEntry( "Query Host", QString::null );
+  
+  QString wins = ((Smb4KCore *)parent())->getWINSServer();
+  QString nmblookup_options = getNmblookupOptions();
+  QString smbclient_options = getSmbclientOptions();
+  
   QString command;
 
   // Use nmblookup or query host to obtain the browse list?
-  if ( m_method == "nmblookup" )
+  if ( method == "nmblookup" )
   {
     command = QString( "nmblookup -M" );
 
-    if ( !m_advNMBOpts.stripWhiteSpace().isEmpty() )
-      command.append( m_advNMBOpts );
+    if ( !nmblookup_options.stripWhiteSpace().isEmpty() )
+      command.append( nmblookup_options );
 
     command.append( " -- - | grep '<01>' | awk '{print $1}'" );
 
-    // Query the WINS server, if available.
-    if ( m_wins.isEmpty() )
+    // Query the WINS server, if available    
+    if ( wins.isEmpty() )
       command.append( " | xargs nmblookup -A" );
     else
-      command.append( QString( " | xargs nmblookup -R -U %1 -A" ).arg( m_wins ) );
+      command.append( QString( " | xargs nmblookup -R -U %1 -A" ).arg( wins ) );
 
     *m_proc << command;
     startProcess( Groups );
   }
-  else if ( m_method == "host" )
+  else if ( method == "host" )
   {
-    command = QString( "smbclient -d1 -N" );
+    command = QString( "smbclient -d1" );
 
-    if ( !m_advSMBOpts.stripWhiteSpace().isEmpty() )
-      command.append( m_advSMBOpts );
-
-    if ( m_hostNeedsAuth )
+    if ( !smbclient_options.stripWhiteSpace().isEmpty() )
     {
-      Smb4KAuthInfo *auth = ((Smb4KCore *)parent())->passwdReader()->getAuth( QString::null, m_hostToStartWith, QString::null );
+      command.append( smbclient_options );
+    }
+
+    if ( auth_for_browse_list )
+    {
+      Smb4KAuthInfo *auth = ((Smb4KCore *)parent())->passwdReader()->getAuth( QString::null, host_to_query, QString::null );
 
       if ( !auth->user().isEmpty() )
       {
         command.append( QString( " -U %1" ).arg( KProcess::quote( auth->user() ) ) );
 
         if ( !auth->password().isEmpty() )
-          command.append( QString( "%%2" ).arg( auth->maskedPassword() ) );
+        {
+          m_proc->setEnvironment( "PASSWD", auth->password() );
+        }
       }
       else
-        command.append( " -U%" );
+      {
+        command.append( " -U %" );
+      }
     }
     else
-      command.append( " -U%" );
+    {
+      command.append( " -U %" );
+    }
 
-    command.append( QString( " -L %1" ).arg( KProcess::quote( m_hostToStartWith ) ) );
+    command.append( QString( " -L %1" ).arg( KProcess::quote( host_to_query ) ) );
 
     *m_proc << command;
+    
     startProcess( QueryHost );
   }
 }
@@ -160,6 +179,8 @@ void Smb4KScanner::getPreview( const QString &workgroup, const QString &host, co
 
 void Smb4KScanner::m_previewShare( const QString &workgroup, const QString &host, const QString &ip, const QString &share, const QString &path )
 {
+  QString smbclient_options = getSmbclientOptions();
+  
   m_workgroup = workgroup;
   m_host = host;
   m_share = share;
@@ -170,7 +191,7 @@ void Smb4KScanner::m_previewShare( const QString &workgroup, const QString &host
   Smb4KAuthInfo *auth = ((Smb4KCore *)parent())->passwdReader()->getAuth( workgroup, host, share );
 
   // Assemble command:
-  command = QString( "smbclient //%1/%2 -d1 -W %3 -N -c 'ls" ).arg( KProcess::quote( host ) ).arg( KProcess::quote( share ) ).arg( KProcess::quote( workgroup ) );
+  command = QString( "smbclient //%1/%2 -d1 -W %3 -c 'ls" ).arg( KProcess::quote( host ) ).arg( KProcess::quote( share ) ).arg( KProcess::quote( workgroup ) );
 
   if ( !path.isEmpty() )
   {
@@ -182,20 +203,31 @@ void Smb4KScanner::m_previewShare( const QString &workgroup, const QString &host
     proc_path = proc_path.replace( QChar( '/' ), QChar( '\\' ) ).local8Bit();
     command.append( " \"" ).append( proc_path ).append( "*\"" );
   }
+  
   command.append( "'" );
   
   if ( !ip.isEmpty() )
+  {
     command.append( QString( " -I %1" ).arg( KProcess::quote( ip ) ) );
+  }
 
-  if ( !m_advSMBOpts.stripWhiteSpace().isEmpty() )
-    command.append( m_advSMBOpts );
+  if ( !smbclient_options.stripWhiteSpace().isEmpty() )
+  {
+    command.append( smbclient_options );
+  }
 
   if ( !auth->user().isEmpty() )
   {
     command.append( QString( " -U %1" ).arg( KProcess::quote( auth->user() ) ) );
     
     if ( !auth->password().isEmpty() )
-      command.append( QString( "%%2" ).arg( auth->maskedPassword() ) );
+    {
+      m_proc->setEnvironment( "PASSWD", auth->password() );
+    }
+  }
+  else
+  {
+    command.append( " -U %" );
   }
 
   m_proc->operator<<( command );
@@ -219,17 +251,24 @@ void Smb4KScanner::getWorkgroupMembers( const QString &workgroup, const QString
 
 void Smb4KScanner::m_scanForWorkgroupMembers( const QString &workgroup, const QString &master, const QString &ip )
 {
+  m_config->setGroup( "Browse Options" );
+  bool master_auth = m_config->readBoolEntry( "Master Browser Auth", false );
+  
+  QString smbclient_options = getSmbclientOptions();
+  
   m_workgroup = workgroup;
   m_host = master;
   m_ip = ip;
 
   // Scan for the host list.
-  QString command = QString( "smbclient -d1 -N" );
+  QString command = QString( "smbclient -d1" );
 
-  if ( !m_advSMBOpts.stripWhiteSpace().isEmpty() )
-    command.append( m_advSMBOpts );
+  if ( !smbclient_options.stripWhiteSpace().isEmpty() )
+  {
+    command.append( smbclient_options );
+  }
 
-  if ( m_mastersAuth )
+  if ( master_auth )
   {
     Smb4KAuthInfo *auth = ((Smb4KCore *)parent())->passwdReader()->getAuth( workgroup, master, QString::null );
 
@@ -238,16 +277,24 @@ void Smb4KScanner::m_scanForWorkgroupMembers( const QString &workgroup, const QS
       command.append( QString( " -U %1" ).arg( KProcess::quote( auth->user() ) ) );
 
       if ( !auth->password().isEmpty() )
-        command.append( QString( "%%2" ).arg( auth->maskedPassword() ) );
+      {
+        m_proc->setEnvironment( "PASSWD", auth->password() );
+      }
     }
     else
-      command.append( QString( " -U%" ) );
+    {
+      command.append( " -U %" );
+    }
   }
   else
-    command.append( QString( " -U%" ) );
+  {
+    command.append( " -U %" );
+  }
 
   if ( !ip.isEmpty() )
+  {
     command.append( QString( " -I %1" ).arg( ip ) );
+  }
     
   command.append( QString( " -W %1 -L %2" ).arg( KProcess::quote( workgroup ) ).arg( KProcess::quote( master ) ) );
 
@@ -272,6 +319,8 @@ void Smb4KScanner::getShares( const QString &workgroup, const QString &host, con
 
 void Smb4KScanner::m_scanForShares( const QString &workgroup, const QString &host, const QString &ip )
 {
+  QString smbclient_options = getSmbclientOptions();
+  
   m_workgroup = workgroup;
   m_host = host;
   m_ip = ip;
@@ -279,20 +328,30 @@ void Smb4KScanner::m_scanForShares( const QString &workgroup, const QString &hos
   Smb4KAuthInfo *auth = ((Smb4KCore *)parent())->passwdReader()->getAuth( workgroup, host, QString::null );
 
   // Assemble the scan command.
-  QString command = QString( "smbclient -d1 -N -W %1" ).arg( KProcess::quote( workgroup ) );
+  QString command = QString( "smbclient -d1 -W %1" ).arg( KProcess::quote( workgroup ) );
   
   if ( !ip.isEmpty() )
+  {
     command.append( QString( " -I %1" ).arg( KProcess::quote( ip ) ) );
+  }
 
-  if ( !m_advSMBOpts.stripWhiteSpace().isEmpty() )
-    command.append( m_advSMBOpts );
+  if ( !smbclient_options.stripWhiteSpace().isEmpty() )
+  {
+    command.append( smbclient_options );
+  }
 
   if ( !auth->user().isEmpty() )
   {
     command.append( QString( " -U %1" ).arg( KProcess::quote( auth->user() ) ) );
     
     if ( !auth->password().isEmpty() )
-      command.append( QString( "%%2" ).arg( auth->maskedPassword() ) );
+    {
+      m_proc->setEnvironment( "PASSWD", auth->password() );
+    }
+  }
+  else
+  {
+    command.append( " -U guest%" );
   }
 
   command.append( " -L " ).append( KProcess::quote( host ) );
@@ -318,19 +377,22 @@ void Smb4KScanner::getInfo( const QString &workgroup, const QString &host, const
 
 void Smb4KScanner::m_scanForInfo( const QString &workgroup, const QString &host, const QString &ip )
 {
+  QString smbclient_options = getSmbclientOptions();
+  
   m_workgroup = workgroup;
   m_host = host;
   
-  // Assemble the scan command.
-  // smbclient >= 3.0 seems to use the current user, if none was specified.
-  // Thus, the -U% is required.
-  QString command = QString( "smbclient -d1 -U% -N -W %1 -L %2" ).arg( KProcess::quote( workgroup ) ).arg( KProcess::quote( host ) );
+  QString command = QString( "smbclient -d1 -U % -W %1 -L %2" ).arg( KProcess::quote( workgroup ) ).arg( KProcess::quote( host ) );
   
   if ( !ip.isEmpty() )
+  {
     command.append( QString( " -I %1" ).arg( KProcess::quote( ip ) ) );
+  }
 
-  if ( !m_advSMBOpts.stripWhiteSpace().isEmpty() )
-    command.append( m_advSMBOpts );
+  if ( !smbclient_options.stripWhiteSpace().isEmpty() )
+  {
+    command.append( smbclient_options );
+  }
 
   m_proc->operator<<( command );
   
@@ -354,44 +416,65 @@ void Smb4KScanner::makeSearch( const QString &host )
 
 void Smb4KScanner::m_searchForHost( const QString &host )
 {
+  m_config->setGroup( "Browse Options" );
+  QString search_method = m_config->readEntry( "Network Search", "nmblookup" );
+
+  if ( search_method == "smbclient" && host.stripWhiteSpace().contains( "." ) == 3 )
+  {
+    emit error( ERROR_IP_CANNOT_BE_USED, QString::null );
+    m_working = false;
+    emit running( SCANNER_STOP, m_working );    
+    return;
+  }
+  
+  QString wins = ((Smb4KCore *)parent())->getWINSServer();
+  QString nmblookup_options = getNmblookupOptions();
+  QString smbclient_options = getSmbclientOptions();
+      
   m_host = host;
   
   QString command;
 
-  if ( m_searchMethod == "nmblookup" )
+  if ( search_method == "nmblookup" )
   {
     command = QString( "nmblookup" );
 
-    if ( !m_advNMBOpts.stripWhiteSpace().isEmpty() )
-      command.append( m_advNMBOpts );
+    if ( !nmblookup_options.stripWhiteSpace().isEmpty() )
+    {
+      command.append( nmblookup_options );
+    }
 
     if ( host.contains( '.', true ) != 3 )
     {
-      // If there is a WINS server, it will be queried for the host.
-      if ( !m_wins.isEmpty() )
-        command.append( QString( " -R -U %1 %2 -S | grep '<00>' | sed -e 's/<00>.*//'" ).arg( m_wins ).arg( m_host ) );
-
-      // If we have none, the program does a normal look-up.
+      if ( !wins.isEmpty() )
+      {
+        command.append( QString( " -R -U %1 %2 -S | grep '<00>' | sed -e 's/<00>.*//'" ).arg( wins ).arg( m_host ) );
+      }
       else
+      {
         command.append( QString( " %1 -S | grep '<00>' | sed -e 's/<00>.*//'" ).arg( m_host ) );
+      }
     }
     else
     {
-      // If there is a WINS server, it will be queried for the host.
-      if ( !m_wins.isEmpty() )
-        command.append( QString( " -R -U %1 %2 -A | grep '<00>' | sed -e 's/<00>.*//'" ).arg( m_wins ).arg( m_host ) );
-
-      // If we have none, the program does a normal look-up.
+      if ( !wins.isEmpty() )
+      {
+        command.append( QString( " -R -U %1 %2 -A | grep '<00>' | sed -e 's/<00>.*//'" ).arg( wins ).arg( m_host ) );
+      }
       else
+      {
         command.append( QString( " %1 -A | grep '<00>' | sed -e 's/<00>.*//'" ).arg( m_host ) );
+      }
     }
   }
   else
   {
-    command = QString( "smbclient -d2 -N -U% -L %1" ).arg( m_host );
+    command = QString( "smbclient -d2 -U % -L %1" ).arg( m_host );
 
-    if ( !m_advSMBOpts.stripWhiteSpace().isEmpty() )
-      command.append( m_advSMBOpts );
+    if ( !smbclient_options.stripWhiteSpace().isEmpty() )
+    {
+      command.append( smbclient_options );
+    }
   }
   
   m_proc->operator<<( command );
@@ -508,13 +591,16 @@ void Smb4KScanner::processWorkgroups()
 
 void Smb4KScanner::processQueryHost()
 {
+  m_config->setGroup( "Browse Options" );
+  QString host_to_query = m_config->readEntry( "Query Host", QString::null );
+  
   QStringList list = QStringList::split( '\n', m_buffer, false );
   
   m_workgroupList.clear();
   
   if ( list.grep( "tree connect failed: NT_STATUS_ACCESS_DENIED" ).count() == 0 && 
        list.grep( "NT_STATUS_LOGON_FAILURE" ).count() == 0 &&
-       list.grep( "Connection to "+m_hostToStartWith+" failed" ).count() == 0 )
+       list.grep( "Connection to "+host_to_query+" failed" ).count() == 0 )
   {
     // Get the line starting with Workgroup.
     uint index( 0 );
@@ -562,19 +648,25 @@ void Smb4KScanner::processQueryHost()
 
       if ( !workgroup.isEmpty() )
       {
-        if ( ((Smb4KCore *)parent())->passwdReader()->askpass( workgroup, m_hostToStartWith, QString::null ) )
+        if ( ((Smb4KCore *)parent())->passwdReader()->askpass( workgroup, host_to_query, QString::null, Smb4KPasswdReader::AccessDenied ) )
         {
           QString *input = new QString( QString( "%1:" ).arg( Init ) );
           m_queue.enqueue( input );
         }
       }
       else
+      {
         emit error( ERROR_GETTING_BROWSELIST, list.grep( "NT_STATUS" ).first() );
+      }
     }
     else if ( list.grep( "NT_STATUS_LOGON_FAILURE" ).count() != 0 )
+    {
       emit error( ERROR_GETTING_BROWSELIST, m_buffer );    
-    else if ( list.grep( "Connection to "+m_hostToStartWith+" failed" ).count() != 0 )
+    }
+    else if ( list.grep( "Connection to "+host_to_query+" failed" ).count() != 0 )
+    {
       emit error( ERROR_GETTING_BROWSELIST, m_buffer ); 
+    }
   }
 }
 
@@ -592,9 +684,10 @@ void Smb4KScanner::processHosts()
     QString errmsg = list.grep( "Connection to" ).first().stripWhiteSpace();
     emit error( ERROR_GETTING_MEMBERS, errmsg );
   }
-  else if ( list.grep( "tree connect failed: NT_STATUS_ACCESS_DENIED" ).count() == 0 && list.grep( "NT_STATUS_LOGON_FAILURE" ).count() != 0  )
+  else if ( m_buffer.contains( "tree connect failed: NT_STATUS_ACCESS_DENIED" ) == 0 
+            && m_buffer.contains( "NT_STATUS_LOGON_FAILURE" ) != 0  )
   {
-    if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, QString::null ) )
+    if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, QString::null, Smb4KPasswdReader::LogonFailure ) )
     {
       QString *input = new QString( QString( "%1:%2:%3" ).arg( Hosts ).arg( m_workgroup ).arg( m_host ) );
       m_queue.enqueue( input );
@@ -662,13 +755,19 @@ void Smb4KScanner::processShares()
       host->setServerString( list[index].section( "Server=[", 1, 1 ).section( "]", 0, 0 ) );
     }
     else if ( list[index].stripWhiteSpace().startsWith( "Sharename" ) )
+    {
       break;
+    }
     else
+    {
       continue;
+    }
   }
   
   emit info( host );
   
+  delete host;
+  
   // Now do the rest:
   if ( list.grep( "NT_STATUS" ).count() != 0 || list.grep( "Connection to" ).count() != 0 || list.grep( "tree connect failed:" ).count() != 0 )
   {
@@ -678,21 +777,36 @@ void Smb4KScanner::processShares()
 
       // The error output of smbclient is a little bit inconsistent:
       if ( errmsg.contains( "NT_STATUS" ) == 0 )
+      {
         errmsg = list.grep( "NT_STATUS" ).first().stripWhiteSpace().section( " ", 0, 0 );
+      }
         
       if ( errmsg == "NT_STATUS_ACCESS_DENIED" || errmsg == "NT_STATUS_LOGON_FAILURE" )
       {
-        if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, QString::null )
+        int state = Smb4KPasswdReader::None;
+        
+        if ( errmsg == "NT_STATUS_ACCESS_DENIED" )
+        {
+          state = Smb4KPasswdReader::AccessDenied;
+        }
+        else if ( errmsg == "NT_STATUS_LOGON_FAILURE" )
+        {
+          state = Smb4KPasswdReader::LogonFailure;
+        }
+        
+        if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, QString::null, state )
 )       {
           m_queue.enqueue( new QString( QString( "%1:%2:%3:%4" ).arg( Shares ).arg( m_workgroup ).arg( m_host ).arg( m_ip ) ) );
         }
       }
       else
+      {
         emit error( ERROR_GETTING_SHARES, list.grep( "NT_STATUS" ).first().stripWhiteSpace() );
+      }
     }
     else if ( list.grep( "tree connect failed:" ).count() != 0 )
     {
-      if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, QString::null ) )
+      if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, QString::null, Smb4KPasswdReader::AccessDenied ) )
       {
         m_queue.enqueue( new QString( QString( "%1:%2:%3:%4" ).arg( Shares ).arg( m_workgroup ).arg( m_host ).arg( m_ip ) ) );
       }
@@ -780,13 +894,13 @@ void Smb4KScanner::processInfo()
     }
     else if ( (*it).contains( "Connection to", true ) != 0 )
     {
-      emit error( ERROR_GETTING_INFO, (*it).stripWhiteSpace() );
-      
       break;
     }
   }
   
   emit info( host );
+  
+  delete host;
 }
 
 
@@ -809,11 +923,24 @@ void Smb4KScanner::processPreview()
       
       // The error output of smbclient is a little bit inconsistent:
       if ( errmsg.contains( "NT_STATUS" ) == 0 )
+      {
         errmsg = list.grep( "NT_STATUS" ).first().stripWhiteSpace().section( " ", -1, -1 );
+      }
       
       if ( errmsg == "NT_STATUS_ACCESS_DENIED" || errmsg == "NT_STATUS_LOGON_FAILURE" )
       {
-        if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, m_share ) )
+        int state = Smb4KPasswdReader::None;
+        
+        if ( errmsg == "NT_STATUS_ACCESS_DENIED" )
+        {
+          state = Smb4KPasswdReader::AccessDenied;
+        }
+        else if ( errmsg == "NT_STATUS_LOGON_FAILURE" )
+        {
+          state = Smb4KPasswdReader::LogonFailure;
+        }
+        
+        if ( ((Smb4KCore *)parent())->passwdReader()->askpass( m_workgroup, m_host, m_share, state ) )
         {
           QString *input = new QString( QString( "%1:%2:%3:%4:%5:%6" ).arg( Preview ).arg( m_workgroup ).arg( m_host ).arg( m_ip ).arg( m_share ).arg( m_path ) );
           m_queue.enqueue( input );
@@ -877,9 +1004,12 @@ void Smb4KScanner::processPreview()
 
 void Smb4KScanner::processSearch()
 {
+  m_config->setGroup( "Browse Options" );
+  QString search_method = m_config->readEntry( "Network Search", "nmblookup" );
+    
   QStringList data = QStringList::split( "\n", m_buffer.stripWhiteSpace(), false );
   
-  if ( m_searchMethod == "nmblookup" )
+  if ( search_method == "nmblookup" )
   {
     if ( !data.isEmpty() )
     {
@@ -909,13 +1039,17 @@ void Smb4KScanner::processSearch()
   {
     if ( data.count() > 1 && !data[1].isEmpty() )
     {
-      QString workgroup = data.grep( "Domain" ).first().section( "Domain=[", 1, 1 ).section( "]", 0, 0 );
-      QString ip = data.grep( "Got a positive name query" ).first().section( "(", 1, 1 ).section( ")", 0, 0 ).stripWhiteSpace();
+      if ( m_buffer.contains( QString( "Connection to %1 failed" ).arg( m_host ) ) != 0 )
+      {
+        emit searchResult( new Smb4KHostItem() );
+      }
+      else
+      {
+        QString workgroup = data.grep( "Domain" ).first().section( "Domain=[", 1, 1 ).section( "]", 0, 0 );
+        QString ip = data.grep( "Got a positive name query" ).first().section( "(", 1, 1 ).section( ")", 0, 0 ).stripWhiteSpace();
 
-      if ( workgroup.stripWhiteSpace().isEmpty() )
-        workgroup = i18n( "UNKNOWN" );
-
-      emit searchResult( new Smb4KHostItem( workgroup, m_host.upper(), QString::null, ip ) );
+        emit searchResult( new Smb4KHostItem( workgroup, m_host.upper(), QString::null, ip ) );
+      }
     }
     else
       emit searchResult( new Smb4KHostItem() );
@@ -923,101 +1057,6 @@ void Smb4KScanner::processSearch()
 }
 
 
-/****************************************************************************
-   Read the options.
-****************************************************************************/
-
-void Smb4KScanner::readOptions()
-{
-  //
-  // Browse options
-  //
-
-  // Determine, which look-up method should be used:
-  m_config->setGroup( "Browse Options" );
-  m_method = m_config->readEntry( "Browse List", "nmblookup" );
-  bool wins = m_config->readBoolEntry( "WINS Support", false );
-
-  // Get the WINS server.
-  if ( wins )
-    m_wins = m_config->readEntry( "WINS Server", QString::null );
-
-  // Get the host to query for the browse list.
-  if ( m_method == "host" || wins == true )
-    m_hostToStartWith = m_config->readEntry( "Query Host", m_wins );
-  else
-    m_hostToStartWith = QString::null;
-
-  m_hostNeedsAuth = m_config->readBoolEntry( "Host Needs Auth", false );
-  m_mastersAuth = m_config->readBoolEntry( "Master Browser Auth", false );
-  m_searchMethod = m_config->readEntry( "Network Search", "nmblookup" );
-
-  //
-  // Advanced Samba options
-  //
-  m_config->setGroup( "Samba" );
-
-  m_advSMBOpts = QString::null;
-  m_advNMBOpts = QString::null;
-
-  if ( !m_config->readEntry( "Client Resolve Order", QString::null ).isEmpty() )
-    m_advSMBOpts.append( QString( " -R \"%1\"" ).arg( m_config->readEntry( "Client Resolve Order", QString::null ) ) );
-
-  if ( !m_config->readEntry( "Port", QString::null ).isEmpty() )
-    m_advSMBOpts.append( QString( " -p %1" ).arg( m_config->readNumEntry( "Port", 139 ) ) );
-
-  if ( !m_config->readEntry( "Client Buffer Size", QString::null ).isEmpty() )
-    m_advSMBOpts.append( QString( " -b %1" ).arg( m_config->readNumEntry( "Client Buffer Size", 65520 ) ) );
-
-  if ( m_config->readBoolEntry( "Use Kerberos", false ) )
-  {
-    m_config->setGroup( "System" );
-
-    if ( m_config->readEntry( "Samba" ).startsWith( "3" ) )
-      m_advSMBOpts.append( " -k" );
-
-    m_config->setGroup( "Samba" );
-  }
-
-  if ( !m_config->readEntry( "NetBIOS Name", QString::null ).isEmpty() )
-  {
-    m_advSMBOpts.append( QString( " -n \"%1\"" ).arg( m_config->readEntry( "NetBIOS Name", QString::null ) ) );
-    m_advNMBOpts.append( QString( " -n \"%1\"" ).arg( m_config->readEntry( "NetBIOS Name", QString::null ) ) );
-  }
-
-  if ( !m_config->readEntry( "NetBIOS Scope", QString::null ).isEmpty() )
-  {
-    m_advSMBOpts.append( QString( " -i \"%1\"" ).arg( m_config->readEntry( "NetBIOS Scope", QString::null ) ) );
-    m_advNMBOpts.append( QString( " -i \"%1\"" ).arg( m_config->readEntry( "NetBIOS Scope", QString::null ) ) );
-  }
-
-  if ( !m_config->readEntry( "Socket Options", QString::null ).isEmpty() )
-  {
-    m_advSMBOpts.append( QString( " -O \"%1\"" ).arg( m_config->readEntry( "Socket Options", QString::null ) ) );
-    m_advNMBOpts.append( QString( " -O \"%1\"" ).arg( m_config->readEntry( "Socket Options", QString::null ) ) );
-  }
-  
-  if ( !m_config->readEntry( "NMB Domain", QString::null ).isEmpty() )
-  {
-    m_config->setGroup( "System" );
-    
-    if ( m_config->readEntry( "Samba" ).startsWith( "3" ) )
-    {
-      m_config->setGroup( "Samba" );
-      m_advNMBOpts.append( QString( " -W \"%1\"" ).arg( m_config->readEntry( "NMB Domain", QString::null ) ) );
-    }
-    
-    m_config->setGroup( "Samba" );
-  }
-  
-  if ( !m_config->readEntry( "NMB Broadcast", QString::null ).isEmpty() )
-    m_advNMBOpts.append( QString( " -B \"%1\"" ).arg( m_config->readEntry( "NMB Broadcast", QString::null ) ) );
-    
-  if ( m_config->readBoolEntry( "NMB Port 137", false ) )
-    m_advNMBOpts.append( " -r" );
-}
-
-
 /****************************************************************************
    Appends an item to the list of workgroups.
 ****************************************************************************/
@@ -1053,6 +1092,80 @@ Smb4KWorkgroupItem *Smb4KScanner::getWorkgroup( const QString &workgroup )
 }
 
 
+/****************************************************************************
+   Returns the options for nmblookup
+****************************************************************************/
+
+const QString Smb4KScanner::getNmblookupOptions()
+{
+  QString options;
+  
+  m_config->setGroup( "Samba" );
+  if ( !m_config->readEntry( "NetBIOS Name", QString::null ).isEmpty() )
+    options.append( QString( " -n \"%1\"" ).arg( m_config->readEntry( "NetBIOS Name", QString::null ) ) );
+
+  if ( !m_config->readEntry( "NetBIOS Scope", QString::null ).isEmpty() )
+    options.append( QString( " -i \"%1\"" ).arg( m_config->readEntry( "NetBIOS Scope", QString::null ) ) );
+
+  if ( !m_config->readEntry( "Socket Options", QString::null ).isEmpty() )
+    options.append( QString( " -O \"%1\"" ).arg( m_config->readEntry( "Socket Options", QString::null ) ) );
+  
+  if ( !m_config->readEntry( "NMB Domain", QString::null ).isEmpty() )
+  {
+    if ( ((Smb4KCore *)parent())->getSambaVersion().startsWith( "3" ) )
+    {
+      options.append( QString( " -W \"%1\"" ).arg( m_config->readEntry( "NMB Domain", QString::null ) ) );
+    }
+  }
+  
+  if ( !m_config->readEntry( "NMB Broadcast", QString::null ).isEmpty() )
+    options.append( QString( " -B \"%1\"" ).arg( m_config->readEntry( "NMB Broadcast", QString::null ) ) );
+    
+  if ( m_config->readBoolEntry( "NMB Port 137", false ) )
+    options.append( " -r" );  
+  
+  return options;
+}
+
+
+/****************************************************************************
+   Returns the options for smbclient
+****************************************************************************/
+
+const QString Smb4KScanner::getSmbclientOptions()
+{
+  QString options;
+  
+  m_config->setGroup( "Samba" );
+
+  if ( !m_config->readEntry( "Client Resolve Order", QString::null ).isEmpty() )
+    options.append( QString( " -R \"%1\"" ).arg( m_config->readEntry( "Client Resolve Order", QString::null ) ) );
+
+  if ( !m_config->readEntry( "Port", QString::null ).isEmpty() )
+    options.append( QString( " -p %1" ).arg( m_config->readNumEntry( "Port", 139 ) ) );
+
+  if ( !m_config->readEntry( "Client Buffer Size", QString::null ).isEmpty() )
+    options.append( QString( " -b %1" ).arg( m_config->readNumEntry( "Client Buffer Size", 65520 ) ) );
+
+  if ( m_config->readBoolEntry( "Use Kerberos", false ) )
+  {
+    if ( ((Smb4KCore *)parent())->getSambaVersion().startsWith( "3" ) )
+      options.append( " -k" );
+  }
+
+  if ( !m_config->readEntry( "NetBIOS Name", QString::null ).isEmpty() )
+    options.append( QString( " -n \"%1\"" ).arg( m_config->readEntry( "NetBIOS Name", QString::null ) ) );
+
+  if ( !m_config->readEntry( "NetBIOS Scope", QString::null ).isEmpty() )
+    options.append( QString( " -i \"%1\"" ).arg( m_config->readEntry( "NetBIOS Scope", QString::null ) ) );
+
+  if ( !m_config->readEntry( "Socket Options", QString::null ).isEmpty() )
+    options.append( QString( " -O \"%1\"" ).arg( m_config->readEntry( "Socket Options", QString::null ) ) );
+  
+  return options;
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // SLOT IMPLEMENTATIONS
 /////////////////////////////////////////////////////////////////////////////
@@ -1139,6 +1252,8 @@ void Smb4KScanner::start()
         m_queue.remove();
       }
     }
+    
+    delete item;
   }
 }
 
