@@ -15,18 +15,38 @@
  *                                                                         *
  ***************************************************************************/
 
- 
+// Qt includes
+#include <qdir.h>
+
+// system includes
+#include <unistd.h>
+#include <sys/types.h>
+  
 // application specific includes
 #include "smb4kshare.h"
 
  
-Smb4KShare::Smb4KShare( const QString &name, const QString &path, const int uid, const int gid ) : m_name(name), m_path(path), m_userinfo( uid, gid )
+Smb4KShare::Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const int uid, const int gid, bool broken ) : m_name(name), m_path(path), m_filesystem( filesystem ), m_userinfo( uid, gid ), m_cifs_login( QString::null ), m_broken( broken )
 {
   //FIXME should throw an exception if one of the param is empty
+  
+  if ( uid != (int)getuid() && gid != (int)getgid() )
+  {
+    m_foreign_mount = true;
+  }
+  else
+  {
+    m_foreign_mount = false;
+  }
 }
 
 
-Smb4KShare::Smb4KShare( const Smb4KShare &s ) : m_name( s.getName() ), m_path( s.getPath() ), m_userinfo( s.getUID(), s.getGID() ) 
+Smb4KShare::Smb4KShare( const QString &name, const QString &path, const QString &filesystem, const QString &username, bool foreign, bool broken ) : m_name(name), m_path(path), m_filesystem( filesystem ), m_userinfo( 0, 0 ), m_cifs_login( username ), m_foreign_mount( foreign ), m_broken( broken )
+{
+}
+
+
+Smb4KShare::Smb4KShare( const Smb4KShare &s ) : m_name( s.getName() ), m_path( s.getPath() ), m_filesystem( s.getFilesystem() ), m_userinfo( s.getUID(), s.getGID() ), m_cifs_login( s.getCIFSLogin() ), m_foreign_mount( s.isForeign() ), m_broken( s.isBroken() )
 {
 }
 
@@ -34,15 +54,8 @@ Smb4KShare::Smb4KShare( const Smb4KShare &s ) : m_name( s.getName() ), m_path( s
 Smb4KShare::~Smb4KShare(){}
 
 
+
 QString Smb4KShare::getName() const
-{
-  QString n( m_name );
-
-  return n.contains( " " ) != 0 ? n.replace( " ", "_" ) : n;
-}
-
-
-QString Smb4KShare::getRealName() const
 {
   return m_name;
 }
@@ -54,6 +67,12 @@ QString Smb4KShare::getPath() const
 }
 
 
+QString Smb4KShare::getCanonicalPath() const
+{
+  return m_broken ? m_path : QDir( m_path ).canonicalPath();
+}
+
+
 int Smb4KShare::getUID() const
 {
   return m_userinfo.uid();
@@ -78,21 +97,33 @@ QString Smb4KShare::getGroup() const
 }
 
 
-void Smb4KShare::clear()
+QString Smb4KShare::getFilesystem() const
 {
-  m_name = QString::null;
-  m_path = QString::null;
+  return m_filesystem;
 }
 
 
-bool Smb4KShare::isEmpty()
+QString Smb4KShare::getCIFSLogin() const
 {
-  if ( m_name == QString::null &&
-       m_path == QString::null )
-  {
-    return true;
-  }
-  else
-    return false;
+  return m_cifs_login;
 }
 
+
+bool Smb4KShare::isForeign() const
+{
+  return m_foreign_mount;
+}
+
+
+bool Smb4KShare::isBroken() const
+{
+  return m_broken;
+}
+
+
+void Smb4KShare::setBroken( bool broken )
+{
+  m_broken = broken;
+}
+
+
