@@ -19,6 +19,7 @@
 #include <qlayout.h>
 #include <qpainter.h>
 #include <qheader.h>
+#include <qvaluelist.h>
 
 // KDE includes
 #include <kapplication.h>
@@ -26,12 +27,16 @@
 #include <kiconloader.h>
 #include <kmessagebox.h>
 #include <kdialogbase.h>
+#include <kglobalsettings.h>
+#include <kactionclasses.h>
+#include <kpopupmenu.h>
 
 // application specific includes
 #include "smb4kbrowserwidget.h"
 #include "../smb4k.h"
 #include "smb4kpreviewdialog.h"
 #include "smb4kprintdialog.h"
+#include "../core/smb4kshare.h"
 
 
 Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KListView( parent, name )
@@ -47,23 +52,20 @@ Smb4KBrowserWidget::Smb4KBrowserWidget( QWidget *parent, const char *name ) : KL
   addColumn( i18n( "IP Address" ), -1 );
   addColumn( i18n( "Comment" ), -1 );
 
-  m_menu = new KPopupMenu( this );
-
   // Widget specific connections.
-  connect( this, SIGNAL( executed( QListViewItem * ) ), SLOT( slotItemExecuted( QListViewItem * ) ) );
-  connect( this, SIGNAL( expanded( QListViewItem * ) ), SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
-  connect( this, SIGNAL( collapsed( QListViewItem * ) ), SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
-  connect( this, SIGNAL( selectionChanged( QListViewItem * ) ), SLOT( slotSelectionChanged( QListViewItem * ) ) );
-  connect( this, SIGNAL( clicked( QListViewItem * ) ), SLOT( slotItemClicked( QListViewItem * ) ) );
+  connect( this, SIGNAL( executed( QListViewItem * ) ),                                SLOT( slotItemExecuted( QListViewItem * ) ) );
+  connect( this, SIGNAL( expanded( QListViewItem * ) ),                                SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
+  connect( this, SIGNAL( collapsed( QListViewItem * ) ),                               SLOT( slotItemExpandedCollapsed( QListViewItem * ) ) );
+  connect( this, SIGNAL( selectionChanged( QListViewItem * ) ),                        SLOT( slotSelectionChanged( QListViewItem * ) ) );
+  connect( this, SIGNAL( clicked( QListViewItem * ) ),                                 SLOT( slotItemClicked( QListViewItem * ) ) );
   connect( this, SIGNAL( rightButtonPressed( QListViewItem *, const QPoint &, int ) ), SLOT( slotRightButtonPressed( QListViewItem *, const QPoint &, int ) ) );
 
   // External connections.
-  connect( sapp->core()->scanner(), SIGNAL( workgroups( const QValueList<Smb4KWorkgroupItem *> & ) ), this, SLOT( slotWorkgroups( const QValueList<Smb4KWorkgroupItem *> & ) ) );
-  connect( sapp->core()->scanner(), SIGNAL( members( const QValueList<Smb4KHostItem *> & ) ), this, SLOT( slotMembers( const QValueList<Smb4KHostItem *> & ) ) );
-  connect( sapp->core()->scanner(), SIGNAL( shares( const QValueList<Smb4KShareItem *> & ) ), this, SLOT( slotShares( const QValueList<Smb4KShareItem *> & ) ) );
-  connect( sapp->core()->ipFinder(), SIGNAL( ip_address( Smb4KHostItem * ) ), this, SLOT( slotAddIPAddress( Smb4KHostItem * ) ) );
-
-  connect( sapp->core(), SIGNAL( runStateChanged() ), this, SLOT( slotCoreRunStateChanged() ) );
+  connect( sapp->core()->scanner(),   SIGNAL( workgroups( const QValueList<Smb4KWorkgroupItem *> & ) ), this, SLOT( slotWorkgroups( const QValueList<Smb4KWorkgroupItem *> & ) ) );
+  connect( sapp->core()->scanner(),   SIGNAL( members( const QValueList<Smb4KHostItem *> & ) ), this,         SLOT( slotMembers( const QValueList<Smb4KHostItem *> & ) ) );
+  connect( sapp->core()->scanner(),   SIGNAL( shares( const QValueList<Smb4KShareItem *> & ) ), this,         SLOT( slotShares( const QValueList<Smb4KShareItem *> & ) ) );
+  connect( sapp->core()->ipFinder(),  SIGNAL( ip_address( Smb4KHostItem * ) ), this,                          SLOT( slotAddIPAddress( Smb4KHostItem * ) ) );
+  connect( sapp->core()->mounter(),   SIGNAL( importedShares() ), this,                                       SLOT( slotMarkShares() ) );
 }
 
 
@@ -80,7 +82,7 @@ void Smb4KBrowserWidget::setOpen( QListViewItem *item, bool open )
     
     sapp->core()->scanner()->getWorkgroupMembers( item->text( Network ), i->master(), i->ip() );
   }
-  else if ( open && item->depth() == 1 && item->childCount() == 0 )
+  else if ( open && item->depth() == 1 ) //&& item->childCount() == 0 )
   {
     sapp->core()->scanner()->getShares( item->parent()->text( Network ), item->text( Network ), item->text( IP ) );
   }
@@ -111,7 +113,7 @@ void Smb4KBrowserWidget::insertItem( Smb4KHostItem *item )
       newItem->setExpandable( true );
     }
     else
-      KMessageBox::sorry( this, i18n( "This computer is already in the list!" ), QString::null, KMessageBox::Notify );
+      KMessageBox::sorry( this, i18n( "This computer is already in the list." ), QString::null, KMessageBox::Notify );
   }
   else
   {
@@ -289,17 +291,18 @@ void Smb4KBrowserWidget::slotSelectionChanged( QListViewItem *item )
 {
   if ( item && item->depth() == 2 )
   {
-    sapp->bookmarksAddBookmark->setEnabled( true );
     sapp->networkAskPass->setEnabled( true );
     
     if ( item->text( Type ).contains( "Printer" ) == 0 )
     {
+      sapp->bookmarksAddBookmark->setEnabled( true );    
       sapp->networkPreview->setEnabled( true );
       sapp->sharesMount->setEnabled( true );
       sapp->networkPrint->setEnabled( false );
     }
     else
     {
+      sapp->bookmarksAddBookmark->setEnabled( false );
       sapp->networkPreview->setEnabled( false );
       sapp->sharesMount->setEnabled( false );
       sapp->networkPrint->setEnabled( true );
@@ -323,91 +326,26 @@ void Smb4KBrowserWidget::slotSelectionChanged( QListViewItem *item )
   }
 
   if ( item )
-    sapp->view()->networkTab()->display( item );
+    sapp->networkTab()->display( item );
 }
 
 
-void Smb4KBrowserWidget::slotRightButtonPressed( QListViewItem *item, const QPoint &point, int col )
+void Smb4KBrowserWidget::slotRightButtonPressed( QListViewItem *, const QPoint &point, int )
 {
-  m_menu->clear();
-
-  // The icon sets:
-  QIconSet reloadSet( SmallIcon( "reload" ), QIconSet::Automatic );
-  QIconSet abortSet( SmallIcon( "stop" ), QIconSet::Automatic );
-  QIconSet searchSet( SmallIcon( "find" ), QIconSet::Automatic );
-  QIconSet bookmarkSet( SmallIcon( "bookmark_add" ), QIconSet::Automatic );
-  QIconSet authSet( SmallIcon( "personal" ), QIconSet::Automatic );
-  QIconSet previewSet( SmallIcon( "view_icon" ), QIconSet::Automatic );  
-  QIconSet printSet( SmallIcon( "printer1" ), QIconSet::Automatic );
-  QIconSet mountSet( SmallIcon( "hdd_mount" ), QIconSet::Automatic );    
+  KActionMenu *menu = new KActionMenu( this, "BrowserPopupMenu" );
+  menu->popupMenu()->insertTitle( SmallIcon( "network" ), i18n( "Network" ), 0, 0 );
+  menu->insert( sapp->actionCollection()->action( "network_rescan" ) );
+  menu->insert( sapp->actionCollection()->action( "abort_action" ) );
+  menu->popupMenu()->insertSeparator( -1 );
+  menu->insert( sapp->actionCollection()->action( "network_search" ) );
+  menu->popupMenu()->insertSeparator( -1 );
+  menu->insert( sapp->actionCollection()->action( "bookmarks_add_bookmark" ) );
+  menu->insert( sapp->actionCollection()->action( "network_askpass" ) );
+  menu->insert( sapp->actionCollection()->action( "network_preview" ) );
+  menu->insert( sapp->actionCollection()->action( "network_print" ) );
+  menu->insert( sapp->actionCollection()->action( "shares_mount" ) );
   
-  // Menu items
-  m_menu->insertTitle( SmallIcon( "network" ), i18n( "Network" ), Title, 0 );
-  m_menu->insertItem( reloadSet, i18n( "Rescan" ), sapp->core()->scanner(), SLOT( slotInit() ), CTRL+Key_R, Rescan, 1 );
-  m_menu->insertItem( abortSet, i18n( "Abort" ), sapp, SLOT( slotAbort() ), CTRL+Key_A, Abort, 2 );
-  m_menu->insertSeparator( 3 );
-  m_menu->insertItem( searchSet, i18n( "Search" ), sapp->view(), SLOT( slotShowSearchTab() ), CTRL+Key_S, Search, 4 );
-  m_menu->insertSeparator( 5 );
-  m_menu->insertItem( bookmarkSet, i18n( "Add Bookmark" ), this, SLOT( slotAddBookmark() ), CTRL+Key_B, Bookmark, 6 );
-  m_menu->insertItem( authSet, i18n( "Authentication" ), this, SLOT( slotAskPass() ), CTRL+Key_T, Password, 7 );
-  m_menu->insertItem( previewSet, i18n( "Preview" ), this, SLOT( slotPreview() ), CTRL+Key_V, Preview, 8 );
-  m_menu->insertItem( printSet, i18n( "Print File" ), this, SLOT( slotPrint() ), CTRL+Key_P, Print, 9 );
-  m_menu->insertItem( mountSet, i18n( "Mount" ), this, SLOT( slotMountShare() ), CTRL+Key_M, Mount, 10 );
-
-  // Enable and disable menu items.
-  if ( sapp->core()->scannerIsRunning() || sapp->core()->mounterIsRunning() )
-  {
-    m_menu->setItemEnabled( Abort, true );
-      
-    if ( sapp->core()->scannerIsRunning() && sapp->core()->scannerState() != SCANNER_STOP )
-    {
-      m_menu->setItemEnabled( Rescan, false );
-    }
-    else
-      m_menu->setItemEnabled( Rescan, true );
-  }
-  else
-  {
-    m_menu->setItemEnabled( Rescan, true );
-    m_menu->setItemEnabled( Abort, false );
-  }
-  
-  if ( item && item->depth() == 2 && col < columns() )
-  {
-    m_menu->setItemEnabled( Bookmark, true );
-    m_menu->setItemEnabled( Password, true );
-    
-    if ( item->text( Type ).contains( "Printer" ) == 0 )
-    {
-      m_menu->setItemEnabled( Preview, true );
-      m_menu->setItemEnabled( Print, false );
-      m_menu->setItemEnabled( Mount, true );
-    }
-    else
-    {
-      m_menu->setItemEnabled( Preview, false );
-      m_menu->setItemEnabled( Print, true );
-      m_menu->setItemEnabled( Mount, false );    
-    }
-  }
-  else if ( item && item->depth() == 1 && col < columns() )
-  {
-    m_menu->setItemEnabled( Bookmark, false );
-    m_menu->setItemEnabled( Password, true );
-    m_menu->setItemEnabled( Preview, false );
-    m_menu->setItemEnabled( Print, false );
-    m_menu->setItemEnabled( Mount, false );
-  }
-  else
-  {
-    m_menu->setItemEnabled( Bookmark, false );
-    m_menu->setItemEnabled( Preview, false );
-    m_menu->setItemEnabled( Print, false );
-    m_menu->setItemEnabled( Mount, false );
-    m_menu->setItemEnabled( Password, false );
-  }
-
-  m_menu->exec( point, 0 );
+  menu->popupMenu()->exec( point, 0 );
 }
 
 
@@ -424,7 +362,7 @@ void Smb4KBrowserWidget::slotItemClicked( QListViewItem *item )
     sapp->sharesMount->setEnabled( false );
     sapp->networkAskPass->setEnabled( false );
 
-    sapp->view()->networkTab()->display( item );
+    sapp->networkTab()->display( item );
   }
 }
 
@@ -432,7 +370,7 @@ void Smb4KBrowserWidget::slotItemClicked( QListViewItem *item )
 void Smb4KBrowserWidget::slotWorkgroups( const QValueList<Smb4KWorkgroupItem *> &list )
 {
   // Clear the network tab completely.
-  sapp->view()->networkTab()->clear( 0 );
+  sapp->networkTab()->clear( 0 );
 
   // Clear the list view:
   clear();
@@ -524,7 +462,7 @@ void Smb4KBrowserWidget::slotShares( const QValueList<Smb4KShareItem *> &list )
         {
           Smb4KBrowserWidgetItem *share = new Smb4KBrowserWidgetItem( host, *it );
           share->setExpandable( false );
-          
+                    
           continue;
         }
         else if ( test != 0 && test->parent() == host )
@@ -540,30 +478,6 @@ void Smb4KBrowserWidget::slotShares( const QValueList<Smb4KShareItem *> &list )
 }
 
 
-void Smb4KBrowserWidget::slotCoreRunStateChanged()
-{
-  // If the menu is shown, enable or disable the
-  // items on the fly.
-  if ( m_menu->isShown() && ( sapp->core()->scannerIsRunning() || sapp->core()->mounterIsRunning() ) )
-  {
-    m_menu->setItemEnabled( Abort, true );
-      
-    if ( sapp->core()->scannerIsRunning() && sapp->core()->scannerState() != SCANNER_STOP )
-    {
-      m_menu->setItemEnabled( Rescan, false );
-    }
-    else
-      m_menu->setItemEnabled( Rescan, true );
-     
-  }
-  else
-  {
-    m_menu->setItemEnabled( Rescan, true );
-    m_menu->setItemEnabled( Abort, false );
-  }
-}
-
-
 void Smb4KBrowserWidget::slotMountShare()
 {
   if ( currentItem() && currentItem()->depth() == 2 )
@@ -588,7 +502,7 @@ void Smb4KBrowserWidget::slotAskPass()
     QString host = currentItem()->text( Network );
     QString share = QString::null;
 
-    sapp->core()->passwdReader()->askpass( workgroup, host, share );
+    sapp->core()->passwdReader()->askpass( workgroup, host, share, Smb4KPasswdReader::NewData );
   }
   else if ( currentItem() && currentItem()->depth() == 2 )
   {
@@ -596,7 +510,7 @@ void Smb4KBrowserWidget::slotAskPass()
     QString host = currentItem()->parent()->text( Network );
     QString share = currentItem()->text( Network );
 
-    sapp->core()->passwdReader()->askpass( workgroup, host, share );
+    sapp->core()->passwdReader()->askpass( workgroup, host, share, Smb4KPasswdReader::NewData );
   }
 }
 
@@ -623,11 +537,15 @@ void Smb4KBrowserWidget::slotAddBookmark()
   if ( currentItem() && currentItem()->depth() == 2 )
   {
     if ( currentItem()->text( Network ).stripWhiteSpace() != "homes" )
-      sapp->core()->bookmarkHandler()->writeBookmark( currentItem()->parent()->text( Network ), currentItem()->text( Network ) );
+    {
+      Smb4KBookmark *bookmark = new Smb4KBookmark( QString( "//%1/%2" ).arg( currentItem()->parent()->text( Network ), currentItem()->text( Network ) ), currentItem()->text( Type ) );
+      sapp->core()->bookmarkHandler()->addBookmark( bookmark );
+    }
     else
     {
       QString share = specifyUser( currentItem()->parent()->text( Network ) );
-      sapp->core()->bookmarkHandler()->writeBookmark( currentItem()->parent()->text( Network ), share );
+      Smb4KBookmark *bookmark = new Smb4KBookmark( QString( "//%1/%2" ).arg( currentItem()->parent()->text( Network ), share ), currentItem()->text( Type ) );
+      sapp->core()->bookmarkHandler()->addBookmark( bookmark );
     }
   }
 }
@@ -658,6 +576,44 @@ void Smb4KBrowserWidget::slotPrint()
 }
 
 
+void Smb4KBrowserWidget::slotMarkShares()
+{
+  m_config->setGroup( "Appearance" );  
+  bool show_all = m_config->readBoolEntry( "Show All Shares", false );
+  
+  QListViewItemIterator it( this );
+  QListViewItem *item;
+    
+  while ((item = it.current()) != 0 )
+  {
+    ++it;
+      
+    if ( item->depth() == 2 )
+    {
+      Smb4KShare *share = sapp->core()->mounter()->findShareByName( QString( "//%1/%2" ).arg( item->parent()->text( Network ), item->text( Network ) ) );
+      
+      if ( sapp->core()->mounter()->isMounted( QString( "//%1/%2" ).arg( item->parent()->text( Network ), item->text( Network ) ) )
+           && ( !share->isForeign() || show_all ) )
+      {
+        if ( !((Smb4KBrowserWidgetItem *)item)->isMounted() )
+        {
+          ((Smb4KBrowserWidgetItem *)item)->setMounted( true );
+        }
+      
+        continue;
+      }
+      else
+      {
+        ((Smb4KBrowserWidgetItem *)item)->setMounted( false );
+        continue;
+      }
+    }
+    else
+      continue;
+  }
+} 
+
+
 
 /***************************************************************************
  *                                                                         *
@@ -666,19 +622,19 @@ void Smb4KBrowserWidget::slotPrint()
  ***************************************************************************/
 
 
-Smb4KBrowserWidgetItem::Smb4KBrowserWidgetItem( QListView *parent, Smb4KWorkgroupItem *item ) : QListViewItem( parent, item->workgroup() ), m_workgroup( m_workgroup ), m_host( NULL ), m_share( NULL )
+Smb4KBrowserWidgetItem::Smb4KBrowserWidgetItem( QListView *parent, Smb4KWorkgroupItem *item ) : QListViewItem( parent, item->workgroup() ), m_workgroup( m_workgroup ), m_host( NULL ), m_share( NULL ), m_mounted( false )
 {
   setPixmap( 0, SmallIcon( "network" ) );
 }
 
 
-Smb4KBrowserWidgetItem::Smb4KBrowserWidgetItem( QListViewItem *parent, Smb4KHostItem *item ) : QListViewItem( parent, item->name(), QString::null, item->ip(), item->comment() ), m_workgroup( NULL ), m_host( item ), m_share( NULL )
+Smb4KBrowserWidgetItem::Smb4KBrowserWidgetItem( QListViewItem *parent, Smb4KHostItem *item ) : QListViewItem( parent, item->name(), QString::null, item->ip(), item->comment() ), m_workgroup( NULL ), m_host( item ), m_share( NULL ), m_mounted( false )
 {
   setPixmap( 0, SmallIcon( "network_local" ) );
 }
   
 
-Smb4KBrowserWidgetItem::Smb4KBrowserWidgetItem( QListViewItem *parent, Smb4KShareItem *item ) : QListViewItem( parent, item->name(), item->type(), QString::null, item->comment() ), m_workgroup( NULL ), m_host( NULL ), m_share( item )
+Smb4KBrowserWidgetItem::Smb4KBrowserWidgetItem( QListViewItem *parent, Smb4KShareItem *item ) : QListViewItem( parent, item->name(), item->type(), QString::null, item->comment() ), m_workgroup( NULL ), m_host( NULL ), m_share( item ), m_mounted( false )
 {
   if ( item->type().contains( "Printer" ) )
     setPixmap( 0, SmallIcon( "printer1" ) );
@@ -692,8 +648,31 @@ Smb4KBrowserWidgetItem::~Smb4KBrowserWidgetItem()
 }
 
 
+void Smb4KBrowserWidgetItem::setMounted( bool mounted )
+{
+  if ( depth() == 2 && text( 1 ) == "Disk" )
+  {
+    m_mounted = mounted;
+    
+    if ( mounted )
+      setPixmap( 0, SmallIcon( "folder_green_open" ) );
+    else
+      setPixmap( 0, SmallIcon( "folder_green" ) );
+  }
+}
+
+
 void Smb4KBrowserWidgetItem::paintCell( QPainter *p, const QColorGroup &cg, int column, int width, int align )
 {
+  QFont f( p->font() );
+  
+  if ( m_mounted )
+    f.setItalic( true );    
+  else
+    f.setItalic( false );
+    
+  p->setFont( f );
+  
   QListViewItem::paintCell( p, cg, column, width, align );
 }
 
