@@ -20,6 +20,7 @@
 #include <qtimer.h>
 #include <qregexp.h>
 #include <qvaluelist.h>
+#include <qmap.h>
 
 #include <qscrollview.h>
 
@@ -30,6 +31,10 @@
 #include <kapplication.h>
 #include <kstandarddirs.h>
 
+// system includes
+#include <sys/types.h>
+#include <sys/stat.h>
+
 // application specific includes
 #include "smb4koptionsdlg.h"
 #include "../smb4k.h"
@@ -53,11 +58,19 @@ Smb4KOptionsDlg::Smb4KOptionsDlg( QWidget *parent, const char *name, bool modal
   setupShareOptions();
   setupAuthOptions();
   setupSambaOptions();
+  setupSuperUserOptions();
   
   readOptions();
-
-  connect( sapp->core()->fileIO(), SIGNAL( doneSUPWriting() ), SLOT( slotDoneSUPWriting() ) );
-  connect( sapp->core()->fileIO(), SIGNAL( noSuper() ), SLOT( slotNoSuper() ) );
+  
+  // Connections this the core
+  connect( sapp->core()->fileIO(),         SIGNAL( no_suid_program() ),       this, SLOT( slotNoSUIDProgram() ) );
+  connect( sapp->core()->fileIO(),         SIGNAL( finished_suid_writing() ), this, SLOT( slotFinishedSUIDWriting() ) );
+  
+  // Others
+  connect( m_sambaPage->m_smbfs,           SIGNAL( toggled( bool ) ),         this, SLOT( slotSambaFileSystem( bool ) ) );
+  connect( m_appearancePage->m_showHidden, SIGNAL( stateChanged( int ) ),     this, SLOT( slotShowHiddenShares( int ) ) );
+  connect( m_networkPage->m_useNMB,        SIGNAL( stateChanged( int ) ),     this, SLOT( slotBrowseListMethod( int ) ) );
+  connect( m_superPage->m_remove,          SIGNAL( clicked() ),               this, SLOT( slotRemoveSuperUserEntries() ) );
 }
 
 
@@ -122,7 +135,7 @@ void Smb4KOptionsDlg::setupShareOptions()
 
 void Smb4KOptionsDlg::setupAuthOptions()
 {
-  QFrame *frame = addPage( i18n( "Authentication" ), i18n( "Authentication" ), DesktopIcon( "personal" ) );
+  QFrame *frame = addPage( i18n( "Authentication" ), i18n( "Authentication" ), DesktopIcon( "identity" ) );
 
   QGridLayout *frameLayout = new QGridLayout( frame );
   frameLayout->setSpacing( 0 );
@@ -191,6 +204,31 @@ void Smb4KOptionsDlg::setupSambaOptions()
 }
 
 
+/****************************************************************************
+   Set up the misc options page.
+****************************************************************************/
+
+void Smb4KOptionsDlg::setupSuperUserOptions()
+{
+  QFrame *frame = addPage( i18n( "Super User" ), i18n( "Super User Privileges" ), DesktopIcon( "misc" ) );
+  
+  QGridLayout *frameLayout = new QGridLayout( frame );
+  frameLayout->setSpacing( 0 );
+  frameLayout->setMargin( 0 );
+
+  QScrollView *container = new QScrollView( frame );
+  container->setResizePolicy( QScrollView::AutoOneFit );
+  container->setFrameShadow( QScrollView::Plain );
+  container->setFrameShape( QScrollView::MenuBarPanel ); 
+  container->setLineWidth( 0 );
+  container->setMidLineWidth( 0 );   
+  m_superPage = new Smb4KSuperUserOptions( container->viewport() );
+  container->addChild( m_superPage, 0, 0 );
+
+  frameLayout->addWidget( container, 0, 0 );  
+}
+
+
 /****************************************************************************
    Saves the options.
 ****************************************************************************/
@@ -215,14 +253,11 @@ bool Smb4KOptionsDlg::saveOptions()
   else
     m_config->writeEntry( "Default Path", m_sharePage->m_defaultPath->lineEdit()->text().append( "/" ) );
   
-  m_config->writeEntry( "Clean Up", m_sharePage->m_cleanUp->isChecked() );
   m_config->writeEntry( "Unmount All", m_sharePage->m_unmountAll->isChecked() );
   m_config->writeEntry( "Mount Recent", m_sharePage->m_mountRecent->isChecked() );
-  m_config->writeEntry( "Allow Unmount Foreign", m_sharePage->m_allowUnmountForeign->isChecked() );
+  m_config->writeEntry( "Unmount Foreign", m_sharePage->m_allowUnmountForeign->isChecked() );
   m_config->writeEntry( "Force Lowercase", m_sharePage->m_lowercase->isChecked() );
-  m_config->writeEntry( "Force Unmount", m_sharePage->m_unmountDead->isChecked() );
-  m_config->writeEntry( "Use SU Privileges", m_sharePage->m_useSuper->isChecked() );
-
+  
   //
   // Browse options
   //
@@ -231,17 +266,22 @@ bool Smb4KOptionsDlg::saveOptions()
   if ( m_networkPage->m_useNMB->isChecked() )
   {
     m_config->writeEntry( "Browse List", "nmblookup" );
-
+    
     if ( !m_networkPage->m_hostName->text().stripWhiteSpace().isEmpty() )
+    {
       m_config->writeEntry( "Query Host", m_networkPage->m_hostName->text().stripWhiteSpace().upper() );
-
-    m_config->writeEntry( "Host Needs Auth", m_networkPage->m_hostNeedsAuth->isChecked() );
+    }
+    else
+    {
+      m_config->deleteEntry( "Query Host" );
+    }
+    m_config->writeEntry( "Auth For Browse List", m_networkPage->m_hostNeedsAuth->isChecked() );
   }
   else if ( m_networkPage->m_useHost->isChecked() )
   {
     m_config->writeEntry( "Browse List", "host" );
     m_config->writeEntry( "Query Host", m_networkPage->m_hostName->text().stripWhiteSpace().upper() );
-    m_config->writeEntry( "Host Needs Auth", m_networkPage->m_hostNeedsAuth->isChecked() );
+    m_config->writeEntry( "Auth For Browse List", m_networkPage->m_hostNeedsAuth->isChecked() );
   }
 
   m_config->writeEntry( "Master Browser Auth", m_networkPage->m_mastersAuth->isChecked() );
@@ -297,17 +337,17 @@ bool Smb4KOptionsDlg::saveOptions()
   m_config->setGroup( "Samba" );
 
   // General
-  if ( !m_sambaPage->m_netbios->text().stripWhiteSpace().isEmpty() && *(sapp->core()->fileIO()->getSMBOptions().find( "netbios name")) != m_sambaPage->m_netbios->text().stripWhiteSpace().upper() )
+  if ( !m_sambaPage->m_netbios->text().stripWhiteSpace().isEmpty() && *(sapp->core()->getGlobalSambaOptions().find( "netbios name")) != m_sambaPage->m_netbios->text().stripWhiteSpace().upper() )
     m_config->writeEntry( "NetBIOS Name", m_sambaPage->m_netbios->text().stripWhiteSpace().upper() );
   else
     m_config->deleteEntry( "NetBIOS Name" );
 
-  if ( !m_sambaPage->m_sockopt->text().stripWhiteSpace().isEmpty() && *(sapp->core()->fileIO()->getSMBOptions().find( "socket options" )) != m_sambaPage->m_sockopt->text().stripWhiteSpace().upper() )
+  if ( !m_sambaPage->m_sockopt->text().stripWhiteSpace().isEmpty() && *(sapp->core()->getGlobalSambaOptions().find( "socket options" )) != m_sambaPage->m_sockopt->text().stripWhiteSpace().upper() )
     m_config->writeEntry( "Socket Options", m_sambaPage->m_sockopt->text().stripWhiteSpace().upper() );
   else
     m_config->deleteEntry( "Socket Options" );
 
-  if ( !m_sambaPage->m_scope->text().stripWhiteSpace().isEmpty() && *(sapp->core()->fileIO()->getSMBOptions().find( "netbios scope")) != m_sambaPage->m_scope->text().stripWhiteSpace().upper() )
+  if ( !m_sambaPage->m_scope->text().stripWhiteSpace().isEmpty() && *(sapp->core()->getGlobalSambaOptions().find( "netbios scope")) != m_sambaPage->m_scope->text().stripWhiteSpace().upper() )
     m_config->writeEntry( "Scope", m_sambaPage->m_scope->text().stripWhiteSpace().upper() );
   else
     m_config->deleteEntry( "NetBIOS Scope" );
@@ -322,7 +362,16 @@ bool Smb4KOptionsDlg::saveOptions()
   else
     m_config->deleteEntry( "Use Kerberos" );
 
-  // smbmount
+  // mount
+  if ( m_sambaPage->m_smbfs->isChecked() )
+  {
+    m_config->writeEntry( "Mount Filesystem", "smbfs" );
+  }
+  else
+  {
+    m_config->writeEntry( "Mount Filesystem", "cifs" );
+  }
+  
   if ( m_sambaPage->m_uid->currentText() != i18n( "default" ) )
     m_config->writeEntry( "Mount UID", m_sambaPage->m_uid->currentText() );
   else
@@ -362,9 +411,12 @@ bool Smb4KOptionsDlg::saveOptions()
     m_config->writeEntry( "Mount LFS", m_sambaPage->m_lfs->isChecked() );
   else
     m_config->deleteEntry( "Mount LFS" );
-
+    
+  m_config->writeEntry( "Mount RSize", m_sambaPage->m_rsize->isChecked() );
+  m_config->writeEntry( "Mount WSize", m_sambaPage->m_wsize->isChecked() );
+    
   // smbclient
-  if ( !m_sambaPage->m_resolve->text().stripWhiteSpace().isEmpty() && *(sapp->core()->fileIO()->getSMBOptions().find( "name resolve order" )) != m_sambaPage->m_resolve->text().stripWhiteSpace().upper() )
+  if ( !m_sambaPage->m_resolve->text().stripWhiteSpace().isEmpty() && *(sapp->core()->getGlobalSambaOptions().find( "name resolve order" )) != m_sambaPage->m_resolve->text().stripWhiteSpace().upper() )
     m_config->writeEntry( "Client Resolve Order", m_sambaPage->m_resolve->text().stripWhiteSpace().upper() );
   else
     m_config->deleteEntry( "Client Resolve Order" );
@@ -376,13 +428,13 @@ bool Smb4KOptionsDlg::saveOptions()
     
   if ( m_sambaPage->m_domain->isEnabled() )
   {
-    if ( !m_sambaPage->m_domain->text().stripWhiteSpace().isEmpty() && m_sambaPage->m_domain->text().stripWhiteSpace().upper() != *(sapp->core()->fileIO()->getSMBOptions().find( "workgroup" )) )
+    if ( !m_sambaPage->m_domain->text().stripWhiteSpace().isEmpty() && m_sambaPage->m_domain->text().stripWhiteSpace().upper() != *(sapp->core()->getGlobalSambaOptions().find( "workgroup" )) )
       m_config->writeEntry( "NMB Domain", m_sambaPage->m_domain->text().stripWhiteSpace().upper() );
     else
       m_config->deleteEntry( "NMB Domain" );
   }
   
-  if ( !m_sambaPage->m_broadcast->text().stripWhiteSpace().isEmpty() && *(sapp->core()->fileIO()->getSMBOptions().find( "interfaces" )) != m_sambaPage->m_broadcast->text().stripWhiteSpace().upper() )
+  if ( !m_sambaPage->m_broadcast->text().stripWhiteSpace().isEmpty() && *(sapp->core()->getGlobalSambaOptions().find( "interfaces" )) != m_sambaPage->m_broadcast->text().stripWhiteSpace().upper() )
     m_config->writeEntry( "NMB Broadcast", m_sambaPage->m_broadcast->text().stripWhiteSpace().upper() );
   else
     m_config->deleteEntry( "NMB Broadcast" );
@@ -390,6 +442,31 @@ bool Smb4KOptionsDlg::saveOptions()
   m_config->writeEntry( "NMB Port 137", m_sambaPage->m_udp->isChecked() );
   
   
+  //
+  // The super user tab
+  //
+  m_config->setGroup( "Super User Privileges" );
+  
+  if ( m_superPage->isEnabled() )
+  {
+    if ( m_superPage->m_super->isChecked() )
+    {
+      m_config->writeEntry( "SUID Program", "super" );
+    }
+    else if ( m_superPage->m_sudo->isChecked() )
+    {
+      m_config->writeEntry( "SUID Program", "sudo" );
+    }
+    m_config->writeEntry( "Force Unmount", m_superPage->m_unmount_dead->isChecked() );
+    m_config->writeEntry( "Run SUID", m_superPage->m_use_suid->isChecked() );
+  }
+  else
+  {
+    m_config->deleteEntry( "Force Unmount" );
+    m_config->deleteEntry( "Run SUID" );
+    m_config->deleteEntry( "SUID Program" );
+  }  
+  
   m_config->sync();
   return true;
 }
@@ -399,60 +476,44 @@ bool Smb4KOptionsDlg::saveOptions()
    Reads the options.
 ****************************************************************************/
 
-
 void Smb4KOptionsDlg::readOptions()
 {
-  //
-  // System options
-  //
-  m_config->setGroup( "System" );
-  QString kernel = m_config->readEntry( "Version", QString::null );
+  int mask = sapp->core()->getUMASK();
+  QString system = sapp->core()->getSystemName();
+  QString kernel = sapp->core()->getKernelVersion();
+  
   int k_major = kernel.section( ".", 0, 0 ).toInt();
   int k_minor = kernel.section( ".", 1, 1 ).toInt();
   int k_rev = kernel.section( ".", 2, 2 ).toInt();
-  QString samba = m_config->readEntry( "Samba", QString::null );
-  int s_major = samba.section( ".", 0, 0 ).toInt();
-  int mask = m_config->readNumEntry( "UMASK" );
-  QString system = m_config->readEntry( "Name", QString::null );
+          
+  int s_major = sapp->core()->getSambaVersion().section( ".", 0, 0 ).toInt();
 
+  m_config->setGroup( "Programs" );
+  QString super = m_config->readPathEntry( "super" );
+  QString sudo = m_config->readPathEntry( "sudo" ); 
+  QString mount_cifs = m_config->readPathEntry( "mount.cifs" ); 
+  
   //
   // Mount options
   //
   m_config->setGroup( "Mount Options" );
   m_sharePage->m_defaultPath->lineEdit()->setText( m_config->readEntry( "Default Path", QDir::homeDirPath().append( "/smb4k/" ) ) );
-  m_sharePage->m_cleanUp->setChecked( m_config->readBoolEntry( "Clean Up", true ) );
   m_sharePage->m_unmountAll->setChecked( m_config->readBoolEntry( "Unmount All", false ) );
   m_sharePage->m_mountRecent->setChecked( m_config->readBoolEntry( "Mount Recent", false ) );
-  m_sharePage->m_allowUnmountForeign->setChecked( m_config->readBoolEntry( "Allow Unmount Foreign", false ) );
+  m_sharePage->m_allowUnmountForeign->setChecked( m_config->readBoolEntry( "Unmount Foreign", false ) );
   m_sharePage->m_lowercase->setChecked( m_config->readBoolEntry( "Force Lowercase", false ) );
-  m_sharePage->m_useSuper->setChecked( m_config->readBoolEntry( "Use SU Privileges", false ) );
-  bool useSuper = m_config->readBoolEntry( "Super", false );
 
-  if ( !useSuper )
-  {
-    m_sharePage->m_unmountDead->setEnabled( false );
-    m_sharePage->m_useSuper->setEnabled( false );
-    m_sharePage->m_removeSUEntries->setEnabled( false );
-  }
-
-  if ( system == "Linux" && k_major >= 2 && ( ( k_minor == 4 && k_rev >= 11 ) || k_minor > 4 ) )
-    m_sharePage->m_unmountDead->setChecked( m_config->readBoolEntry( "Force Unmount", false ) );
-  else
-  {
-    m_sharePage->m_unmountDead->setChecked( false );
-    m_sharePage->m_unmountDead->setEnabled( false );
-  }
 
   //
   // Browse options
   //
   m_config->setGroup( "Browse Options" );
   // Toggle the browse list buttons.
-  bool winsSupport = m_config->readBoolEntry( "WINS Support", false );
   QString browselist = m_config->readEntry( "Browse List", "nmblookup" );
-  QString wins =  m_config->readEntry( "WINS Server", QString::null );
-  QString host = m_config->readEntry( "Query Host", wins );
+  QString host = m_config->readEntry( "Query Host", QString::null );
 
+  QString wins = sapp->core()->getWINSServer();
+  
   if ( browselist.stripWhiteSpace() == "nmblookup" )
   {
     m_networkPage->m_useNMB->setChecked( true );
@@ -468,10 +529,10 @@ void Smb4KOptionsDlg::readOptions()
   if ( !host.isEmpty() )
     m_networkPage->m_hostName->setText( host );
 
-  if ( winsSupport )
+  if ( !wins.isEmpty() )
     m_networkPage->m_winsYesNo->setText( wins );
 
-  m_networkPage->m_hostNeedsAuth->setChecked( m_config->readBoolEntry( "Host Needs Auth", false ) );
+  m_networkPage->m_hostNeedsAuth->setChecked( m_config->readBoolEntry( "Auth For Browse List", false ) );
   m_networkPage->m_mastersAuth->setChecked( m_config->readBoolEntry( "Master Browser Auth", false ) );
 
   if ( m_config->readEntry( "Network Search", "nmblookup" ) == "nmblookup" )
@@ -484,10 +545,14 @@ void Smb4KOptionsDlg::readOptions()
   // Appearance
   //
   m_config->setGroup( "Appearance" );
-  m_appearancePage->m_showHidden->setChecked( m_config->readBoolEntry( "Show Hidden", true ) );
+  bool show_hidden = m_config->readBoolEntry( "Show Hidden", true );
+  m_appearancePage->m_showHidden->setChecked( show_hidden );
 
-  // The program has to be notified that the toggle state changed.
-  m_appearancePage->init();
+  if ( !show_hidden )
+  {
+    m_appearancePage->m_showIPC->setEnabled( false );
+    m_appearancePage->m_showAdmin->setEnabled( false );
+  }
 
   m_appearancePage->m_showIPC->setChecked( m_config->readBoolEntry( "Show IPC", false ) );
   m_appearancePage->m_showAdmin->setChecked( m_config->readBoolEntry( "Show ADMIN", false ) );
@@ -497,14 +562,22 @@ void Smb4KOptionsDlg::readOptions()
   m_appearancePage->m_showComment->setChecked( m_config->readBoolEntry( "Show Comment", true ) );
            
   if ( m_config->readEntry( "Show Shares", "icons" ) == "icons" )
+  {
     m_appearancePage->m_shareview->setChecked( false );
+  }
   else if ( m_config->readEntry( "Show Shares", "icons" ) == "list" )
+  {
     m_appearancePage->m_shareview->setChecked( true );
+  }
   
   if ( !m_config->readBoolEntry( "Show Mount Point", false ) )
+  {
     m_appearancePage->m_description->setChecked( false );
+  }
   else
+  {
     m_appearancePage->m_description->setChecked( true );
+  }
     
   m_appearancePage->m_external->setChecked( m_config->readBoolEntry( "Show All Shares", false ) );
   
@@ -554,9 +627,9 @@ void Smb4KOptionsDlg::readOptions()
   // Samba options
   //
   m_config->setGroup( "Samba" );
-  m_sambaPage->m_netbios->setText( m_config->readEntry( "NetBIOS Name", *(sapp->core()->fileIO()->getSMBOptions().find( "netbios name" )) ) );
-  m_sambaPage->m_sockopt->setText( m_config->readEntry( "Socket Options", *(sapp->core()->fileIO()->getSMBOptions().find( "socket options" )) ) );
-  m_sambaPage->m_scope->setText( m_config->readEntry( "NetBIOS Scope", *(sapp->core()->fileIO()->getSMBOptions().find( "netbios scope" )) ) );
+  m_sambaPage->m_netbios->setText( m_config->readEntry( "NetBIOS Name", *(sapp->core()->getGlobalSambaOptions().find( "netbios name" )) ) );
+  m_sambaPage->m_sockopt->setText( m_config->readEntry( "Socket Options", *(sapp->core()->getGlobalSambaOptions().find( "socket options" )) ) );
+  m_sambaPage->m_scope->setText( m_config->readEntry( "NetBIOS Scope", *(sapp->core()->getGlobalSambaOptions().find( "netbios scope" )) ) );
   m_sambaPage->m_port->setValue( m_config->readNumEntry( "Port", 139 ) );
 
   if ( s_major >= 3 )
@@ -572,6 +645,24 @@ void Smb4KOptionsDlg::readOptions()
     m_sambaPage->m_unicode->setEnabled( false );
   }
   
+  QString filesystem = m_config->readEntry( "Mount Filesystem", "smbfs" );
+  
+  if ( filesystem == "smbfs" || mount_cifs.isEmpty() )
+  {
+    m_sambaPage->m_smbfs->setChecked( true );
+    slotSambaFileSystem( true );
+  }
+  else
+  {
+    m_sambaPage->m_cifs->setChecked( true );
+    slotSambaFileSystem( false );
+  }
+  
+  if ( mount_cifs.isEmpty() )
+  {
+    m_sambaPage->m_cifs->setEnabled( false );
+  }
+  
   QValueList<Smb4KUser *> list = sapp->core()->fileIO()->getUsers();
   QStringList uid, gid;
     
@@ -579,11 +670,15 @@ void Smb4KOptionsDlg::readOptions()
   {
     // Put the UIDs in the respective combo box.
     if ( uid.find( QString( "%1" ).arg( (*it)->uid() ) ) == uid.end() )
+    {
       uid.append( QString( "%1" ).arg( (*it)->uid() ) );
+    }
     
     // Put the GIDs in the respective combo box.
     if ( gid.find( QString( "%1" ).arg( (*it)->gid() ) ) == gid.end() )
+    {
       gid.append( QString( "%1" ).arg( (*it)->gid() ) );    
+    }
   }
     
   uid.sort();
@@ -619,18 +714,63 @@ void Smb4KOptionsDlg::readOptions()
   else
     m_sambaPage->m_ttl->setEnabled( false );
 
+  m_sambaPage->m_rsize->setChecked( m_config->readBoolEntry( "Mount RSize", false ) );
+  m_sambaPage->m_wsize->setChecked( m_config->readBoolEntry( "Mount WSize", false ) );
+
   m_sambaPage->m_buffer->setValue( m_config->readNumEntry( "Client Buffer Size", 65520 ) );
-  m_sambaPage->m_resolve->setText( m_config->readEntry( "Client Resolve Order", *(sapp->core()->fileIO()->getSMBOptions().find( "name resolve order" )) ) );
+  m_sambaPage->m_resolve->setText( m_config->readEntry( "Client Resolve Order", *(sapp->core()->getGlobalSambaOptions().find( "name resolve order" )) ) );
   
   if ( s_major >= 3 )
     m_sambaPage->m_domain->setEnabled( true );
   else
     m_sambaPage->m_domain->setEnabled( false );
 
-  m_sambaPage->m_domain->setText( m_config->readEntry( "NMB Domain", *(sapp->core()->fileIO()->getSMBOptions().find( "workgroup" )) ) );
-  m_sambaPage->m_broadcast->setText( m_config->readEntry( "NMB Broadcast", *(sapp->core()->fileIO()->getSMBOptions().find( "interfaces" )) ) );
+  m_sambaPage->m_domain->setText( m_config->readEntry( "NMB Domain", *(sapp->core()->getGlobalSambaOptions().find( "workgroup" )) ) );
+  m_sambaPage->m_broadcast->setText( m_config->readEntry( "NMB Broadcast", *(sapp->core()->getGlobalSambaOptions().find( "interfaces" )) ) );
   m_sambaPage->m_udp->setChecked( m_config->readBoolEntry( "NMB Port 137", false ) );
   
+  //
+  // Super user options
+  //
+  m_config->setGroup( "Super User Privileges" );
+  
+  if ( super.isEmpty() && sudo.isEmpty() )
+  {
+    m_superPage->setEnabled( false );
+  }
+  else if ( !super.isEmpty() && sudo.isEmpty() )
+  {
+    m_superPage->m_super->setChecked( true );
+    m_superPage->m_sudo->setEnabled( false );
+  }
+  else if ( super.isEmpty() && !sudo.isEmpty() )
+  {
+    m_superPage->m_sudo->setChecked( true );
+    m_superPage->m_super->setEnabled( false );
+  }
+  else
+  {
+    if ( m_config->readEntry( "SUID Program", "super" ) == "super" )
+    {
+      m_superPage->m_super->setChecked( true );
+    }
+    else if ( m_config->readEntry( "SUID Program", "super" ) == "sudo" )
+    {
+      m_superPage->m_sudo->setChecked( true );
+    }
+  }
+
+  if ( system == "Linux" && k_major >= 2 && ( ( k_minor == 4 && k_rev >= 11 ) || k_minor > 4 ) )
+  {
+    m_superPage->m_unmount_dead->setChecked( m_config->readBoolEntry( "Force Unmount", false ) );
+  }
+  else
+  {
+    m_superPage->m_unmount_dead->setChecked( false );
+    m_superPage->m_unmount_dead->setEnabled( false );
+  }  
+
+  m_superPage->m_use_suid->setChecked( m_config->readBoolEntry( "Run SUID", false ) );
 }
 
 
@@ -733,18 +873,30 @@ void Smb4KOptionsDlg::slotOk()
 {
   if ( check() )
   {
-    if ( m_sharePage->m_unmountDead->isChecked() || m_sharePage->m_useSuper->isChecked() )
+    if ( m_superPage->m_unmount_dead->isChecked() || m_superPage->m_use_suid->isChecked() )
     {
       m_button = "OK";
-      // Modify the /etc/super.tab file to gain super user privileges for
-      // mounting/unmounting shares.
-      sapp->core()->fileIO()->writeSuperUserEntries( m_sharePage->m_unmountDead->isChecked(), m_sharePage->m_useSuper->isChecked(), false );
+      
+      QString prog;
+      
+      if ( m_superPage->m_super->isChecked() )
+        prog = "super";
+      else if ( m_superPage->m_sudo->isChecked() )
+        prog = "sudo";
+      
+      if ( sapp->core()->fileIO()->writeSuperUserEntries( prog ) )
+      {
+        if ( saveOptions() )
+        {
+          emit read();
+        }
+      }
     }
     else
     {
       if ( saveOptions() )
       {
-        sapp->readOptions();
+        emit read();
         accept();      
       }
     }
@@ -756,48 +908,117 @@ void Smb4KOptionsDlg::slotApply()
 {
   if ( check() )
   {
-    if ( m_sharePage->m_unmountDead->isChecked() || m_sharePage->m_useSuper->isChecked() )
+    if ( m_superPage->m_unmount_dead->isChecked() || m_superPage->m_use_suid->isChecked() )
     {
       m_button = "Apply";
-      // Modify the /etc/super.tab file to gain super user privileges for
-      // mounting/unmounting shares.
-      sapp->core()->fileIO()->writeSuperUserEntries( m_sharePage->m_unmountDead->isChecked(), m_sharePage->m_useSuper->isChecked(), false );
+      
+      QString prog;
+      
+      if ( m_superPage->m_super->isChecked() )
+        prog = "super";
+      else if ( m_superPage->m_sudo->isChecked() )
+        prog = "sudo";
+      
+      if ( sapp->core()->fileIO()->writeSuperUserEntries( prog ) )
+      {
+        if ( saveOptions() )
+        {
+          emit read();
+        }
+      }      
     }
     else
     {
       if ( saveOptions() )
       {
-        sapp->readOptions();
+        emit read();
       }
     }
   }
 }
 
 
-void Smb4KOptionsDlg::slotDoneSUPWriting()
+void Smb4KOptionsDlg::slotNoSUIDProgram()
 {
-  if ( m_button == "Apply" )
+  m_superPage->m_unmount_dead->setChecked( false );
+  m_superPage->m_use_suid->setChecked( false );
+}
+
+
+void Smb4KOptionsDlg::slotFinishedSUIDWriting()
+{
+  if ( m_button == "OK" )
+    accept();
+}
+
+/****************************************************************************
+   SLOTS FOR WIDGETS
+****************************************************************************/
+
+void Smb4KOptionsDlg::slotSambaFileSystem( bool on )
+{
+  if ( on )
   {
-    if ( saveOptions() )
-      sapp->readOptions();
+    ((QGroupBox *)m_sambaPage->child( "SambaCharsetCodepage", "QGroupBox" ))->setEnabled( true );
+    ((QLabel *)m_sambaPage->child( "SambaCachingTime", "QLabel" ))->setEnabled( true );
+    m_sambaPage->m_ttl->setEnabled( true );
+    m_sambaPage->m_unicode->setEnabled( true );
+    m_sambaPage->m_lfs->setEnabled( true );
+    m_sambaPage->m_rsize->setEnabled( false );
+    m_sambaPage->m_wsize->setEnabled( false );
   }
-  else if ( m_button == "OK" )
+  else 
   {
-    if ( saveOptions() )
-    {
-      sapp->readOptions();
-      accept();    
-    }
+    ((QGroupBox *)m_sambaPage->child( "SambaCharsetCodepage", "QGroupBox" ))->setEnabled( false );
+    ((QLabel *)m_sambaPage->child( "SambaCachingTime", "QLabel" ))->setEnabled( false );
+    m_sambaPage->m_ttl->setEnabled( false );
+    m_sambaPage->m_unicode->setEnabled( false );
+    m_sambaPage->m_lfs->setEnabled( false );
+    m_sambaPage->m_rsize->setEnabled( true );
+    m_sambaPage->m_wsize->setEnabled( true );
   }
 }
 
 
-void Smb4KOptionsDlg::slotNoSuper()
+void Smb4KOptionsDlg::slotShowHiddenShares( int state )
 {
-  m_sharePage->m_unmountDead->setChecked( false );
-  m_sharePage->m_useSuper->setChecked( false );
+  if ( state == QCheckBox::On )
+  {
+    m_appearancePage->m_showIPC->setEnabled( true );
+    m_appearancePage->m_showAdmin->setEnabled( true );
+  }
+  else if ( state == QCheckBox::Off )
+  {
+    m_appearancePage->m_showIPC->setEnabled( false );
+    m_appearancePage->m_showAdmin->setEnabled( false );
+  }
 }
 
 
+void Smb4KOptionsDlg::slotBrowseListMethod( int state )
+{
+  if ( state == QRadioButton::On )
+  {
+    m_networkPage->m_hostLabel->setEnabled( false );
+    m_networkPage->m_hostName->setEnabled( false );
+    m_networkPage->m_hostNeedsAuth->setEnabled( false );
+  }
+  else if ( state == QRadioButton::Off )
+  {
+    m_networkPage->m_hostLabel->setEnabled( true );
+    m_networkPage->m_hostName->setEnabled( true );
+    m_networkPage->m_hostNeedsAuth->setEnabled( true );
+  }
+}
+
+
+void Smb4KOptionsDlg::slotRemoveSuperUserEntries()
+{
+  if ( sapp->core()->fileIO()->removeSuperUserEntries() )
+  {
+    m_superPage->m_unmount_dead->setChecked( false );
+    m_superPage->m_use_suid->setChecked( false );
+  }
+}
+
 #include "smb4koptionsdlg.moc"
-
